"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Sequence, TypeVar
from pandas._typing import npt

_SparseIndexT = TypeVar("_SparseIndexT", bound=SparseIndex)
class SparseIndex:
    length: int
    npoints: int
    def __init__(self) -> None:
        ...
    
    @property
    def ngaps(self) -> int:
        ...
    
    @property
    def nbytes(self) -> int:
        ...
    
    @property
    def indices(self) -> npt.NDArray[np.int32]:
        ...
    
    def equals(self, other) -> bool:
        ...
    
    def lookup(self, index: int) -> np.int32:
        ...
    
    def lookup_array(self, indexer: npt.NDArray[np.int32]) -> npt.NDArray[np.int32]:
        ...
    
    def to_int_index(self) -> IntIndex:
        ...
    
    def to_block_index(self) -> BlockIndex:
        ...
    
    def intersect(self: _SparseIndexT, y_: SparseIndex) -> _SparseIndexT:
        ...
    
    def make_union(self: _SparseIndexT, y_: SparseIndex) -> _SparseIndexT:
        ...
    


class IntIndex(SparseIndex):
    indices: npt.NDArray[np.int32]
    def __init__(self, length: int, indices: Sequence[int], check_integrity: bool = ...) -> None:
        ...
    


class BlockIndex(SparseIndex):
    nblocks: int
    blocs: np.ndarray
    blengths: np.ndarray
    def __init__(self, length: int, blocs: np.ndarray, blengths: np.ndarray) -> None:
        ...
    


def make_mask_object_ndarray(arr: npt.NDArray[np.object_], fill_value) -> npt.NDArray[np.bool_]:
    ...

def get_blocks(indices: npt.NDArray[np.int32]) -> tuple[npt.NDArray[np.int32], npt.NDArray[np.int32]]:
    ...




"""
This type stub file was generated by pyright.
"""

import dataclasses
from typing import Hashable, Literal

"""
Provide basic components for groupby. These definitions
hold the allowlist of methods that are exposed on the
SeriesGroupBy and the DataFrameGroupBy objects.
"""
@dataclasses.dataclass(order=True, frozen=True)
class OutputKey:
    label: Hashable
    position: int
    ...


plotting_methods = ...
common_apply_allowlist = ...
series_apply_allowlist: frozenset[str] = ...
dataframe_apply_allowlist: frozenset[str] = ...
cythonized_kernels = ...
reduction_kernels = ...
def maybe_normalize_deprecated_kernels(kernel) -> Literal["bfill", "ffill"]:
    ...

transformation_kernels = ...
groupby_other_methods = ...
transform_kernel_allowlist = ...



# BoARIO : The Adaptative Regional Input Output model in python.
# Copyright (C) 2022  Samuel Juhel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import re
import sys
import subprocess
import pandas as pd

module_path = os.path.abspath(os.path.join("../"))
if module_path not in sys.path:
    sys.path.append(module_path)

module_path = os.path.abspath(os.path.join("./"))
if module_path not in sys.path:
    sys.path.append(module_path)

import boario
from boario.simulation import Simulation
import json
import pathlib
import logging
import pickle
import argparse

def dist_is_editable():
   """Is distribution an editable install?"""
   for pth in boario.__path__:
       if "site-packages" in pth:
           return False
   return True

def get_git_describe() -> str:
    return subprocess.check_output(['git', 'describe', '--tags']).decode('ascii').strip()

def run(region, params, psi, inv_tau, stype, rtype, flood_dmg, mrios_path, output_dir, flood_gdp_file, event_file, mrio_params, alt_inv_dur=None):
    with open(params) as f:
        scriptLogger.info("Loading simulation params template from {}".format(params))
        params_template = json.load(f)
    params_template["output_dir"] = output_dir
    params_template["mrio_params_file"] = mrio_params
    #with open(flood_gdp_file) as f:
    #    flood_gdp_share = json.load(f)
    flood_gdp_df = pd.read_parquet(flood_gdp_file)

    with open(event_file) as f:
        scriptLogger.info("Loading event template from {}".format(event_file))
        event_template = json.load(f)

    mrio_path = pathlib.Path(mrios_path)

    with mrio_path.open("rb") as f:
        mrio = pickle.load(f)

    event = event_template.copy()
    sim_params = params_template.copy()
    scriptLogger.info("Setting psi parameter to {}".format(float(psi.replace("_","."))))
    sim_params["psi_param"] = float(psi.replace("_","."))
    sim_params["inventory_restoration_tau"] = inv_tau
    scriptLogger.info("Setting inventory restoration time to {}".format(inv_tau))

    #TODO remove this ?
    value_added = (mrio.x.T - mrio.Z.sum(axis=0))
    value_added = value_added.reindex(sorted(value_added.index), axis=0) #type: ignore
    value_added = value_added.reindex(sorted(value_added.columns), axis=1)
    value_added[value_added < 0] = 0.0
    gdp_df = value_added.groupby("region",axis=1).sum().T["indout"]
    if mrio.unit.unit.unique()[0] != "M.EUR" :
        scriptLogger.warning("MRIO unit appears to not be 'M.EUR'; but {} instead, which is not yet implemented. Contact the dev !".format(mrio.unit.unit.unique()[0]))
    else:
        gdp_df = gdp_df*(10**6)

    #TODO : Finish this
    # if stype == "Subregions":
    #     scriptLogger.info("Subregions run detected !")
    #     if "sliced" not in str(mrio_path):
    #         raise ValueError("mrio {} seems not to contain subregions (sliced is not present in its name)".format(str(mrio_path)))
    #     else:
    #         mrio_rgxp = re.compile(r"(?P<region>[A-Z]{2})_sliced_in_(?P<split_number>[0-9]+)")
    #         if (mrio_match := mrio_rgxp.match(mrio_path.stem)) is None:
    #             raise ValueError("MRIO ({}) is not valid in this context".format(str(mrio_path.stem)))
    #         else:
    #             splited_region = mrio_match['region']
    #             split_number = int(mrio_match['split_number'])
    #         region_rgxp = re.compile(r"(?P<main_region>[A-Z]{2,3})-(?P<subregion>(?P<sregionname>[A-Z]{2,3})_?(?P<n>\d+)|all|one)")
    #         if (match := region_rgxp.match(region)) is None:
    #             raise ValueError("Impacted region ({}) is not valid in this context".format(str(region)))
    #         else:
    #             if match['main_region'] != splited_region:
    #                 raise ValueError("Impacted region ({}) is different from the splited region ({})".format(str(region, splited_region)))
    #             event["main_region"] = match['main_region']
    #             if match['subregion'] == "all":
    #                 event["aff_regions"] = [event["main_region"]+"_"+str(i) for i in range(split_number)]
    #             elif match['subregion'] == "one":
    #                 event["aff_regions"] = event["main_region"]+"_1"
    #             else:
    #                 event["aff_regions"] = match['sregionname']+"_"+match['n']
    # elif stype == "RoW":
    #     pass
    # elif stype== "Full":
    #     pass
    # else:
    #     raise ValueError("Simulation type {} is incorrect".format(stype))

    scriptLogger.info("Done !")
    scriptLogger.info("Main storage dir is : {}".format(pathlib.Path(params_template["output_dir"]).resolve()))
    if rtype == "int":
        event_row = flood_gdp_df.loc[(flood_gdp_df['class'] == flood_dmg) & (flood_gdp_df['EXIO3_region'] == region)]
        if event_row.empty:
            raise ValueError("This tuple of region / flood class ({},{}) does not have a representative event (it is likely a duplicate of another class that was removed)".format(region,flood_dmg))
        dmg_as_gdp_share = float(event_row['share of GVA used as ARIO input'])
        total_direct_dmg = dmg_as_gdp_share * gdp_df[region] #float(event_row['total_dmg'])
        duration = int(event_row['duration'])
        scriptLogger.info("Setting flood duration to {}".format(duration))
        event["duration"] = duration
        event["r_dmg"] = dmg_as_gdp_share
        event["kapital_damage"] = total_direct_dmg
    elif rtype == "raw":
        dmg = flood_dmg
        event["r_dmg"] = float(flood_dmg) / float(gdp_df[region])
        scriptLogger.info("Damages represent : {}/{} = {} of the region GDP".format(flood_dmg, gdp_df[region], event['r_dmg']))
        event["kapital_damage"] = float(dmg)
    else:
        raise ValueError("Run damage type {} is incorrect".format(rtype))

    event["aff_regions"] = region
    scriptLogger.info("Setting aff_regions to {}".format(region))
    sim_params["output_dir"] = output_dir
    if alt_inv_dur:
        sim_params["results_storage"] = region+"_type_"+stype+"_qdmg_"+rtype+"_"+flood_dmg+"_Psi_"+psi+"_inv_tau_"+str(sim_params["inventory_restoration_tau"])+"_inv_time_"+str(int(alt_inv_dur))
    else:
        sim_params["results_storage"] = region+"_type_"+stype+"_qdmg_"+rtype+"_"+flood_dmg+"_Psi_"+psi+"_inv_tau_"+str(sim_params["inventory_restoration_tau"])

    if event["shock_type"] == "kapital_destroyed_recover":
        event["recovery_time"] = sim_params["rebuild_tau"]
    sim = Simulation(sim_params, mrio_path, modeltype=sim_params['model_type'])
    if alt_inv_dur:
        sim.model.change_inv_duration(alt_inv_dur)

    print(event)
    sim.read_events_from_list([event])
    try:
        scriptLogger.info("Model ready, looping")
        sim.loop(progress=False)
    except Exception:
        scriptLogger.exception("There was a problem:")

parser = argparse.ArgumentParser(description="Produce indicators from one run folder")
parser.add_argument("region", type=str, help="The region to run")
parser.add_argument("params", type=str, help="The params file")
parser.add_argument("psi", type=str, help="The psi parameter")
parser.add_argument("inv_tau", type=str, help="The inventory restoration parameter")
parser.add_argument("stype", type=str, help="The type (RoW or Full) simulation to run")
parser.add_argument("rtype", type=str, help="The damage type (raw or int)")
parser.add_argument("flood_dmg", type=str, help="The flood damage/intensity to run")
parser.add_argument("mrios_path", type=str, help="The mrios path")
parser.add_argument("output_dir", type=str, help="The output directory")
parser.add_argument("flood_gdp_file", type=str, help="The share of gdp impacted according to flood distribution file")
parser.add_argument("event_file", type=str, help="The event template file")
parser.add_argument("mrio_params", type=str, help="The mrio parameters file")
parser.add_argument("alt_inv_dur", type=str, help="The optional alternative main inventory duration", nargs="?", default=None)

if __name__ == "__main__":
    args = parser.parse_args()
    logFormatter = logging.Formatter("%(asctime)s [%(levelname)-5.5s] %(name)s %(message)s", datefmt="%H:%M:%S")
    scriptLogger = logging.getLogger("generic_run - {}_{}_{}_{}_{}_{}".format(args.region, args.psi, args.inv_tau, args.stype, args.rtype, args.flood_dmg))
    consoleHandler = logging.StreamHandler()
    consoleHandler.setFormatter(logFormatter)
    scriptLogger.addHandler(consoleHandler)
    scriptLogger.setLevel(logging.INFO)
    scriptLogger.propagate = False
    scriptLogger.info("You are running the following version of BoARIO : {}".format(boario.__version__))
    scriptLogger.info("You are using BoARIO in editable install mode : {}".format(dist_is_editable()))
    scriptLogger.info("You are using BoARIO in editable install mode : {}".format(boario.__path__))
    scriptLogger.info("=============== STARTING RUN ================")
    run(args.region, args.params, args.psi, int(args.inv_tau), args.stype, args.rtype, args.flood_dmg, args.mrios_path, args.output_dir, args.flood_gdp_file, args.event_file, args.mrio_params, args.alt_inv_dur)



"""
This type stub file was generated by pyright.
"""

from pandas._typing import FilePath, ReadBuffer, StorageOptions, WriteBuffer
from pandas.util._decorators import doc
from pandas import DataFrame
from pandas.core.shared_docs import _shared_docs

""" parquet compat """
def get_engine(engine: str) -> BaseImpl:
    """return our implementation"""
    ...

class BaseImpl:
    @staticmethod
    def validate_dataframe(df: DataFrame) -> None:
        ...
    
    def write(self, df: DataFrame, path, compression, **kwargs):
        ...
    
    def read(self, path, columns=..., **kwargs) -> DataFrame:
        ...
    


class PyArrowImpl(BaseImpl):
    def __init__(self) -> None:
        ...
    
    def write(self, df: DataFrame, path: FilePath | WriteBuffer[bytes], compression: str | None = ..., index: bool | None = ..., storage_options: StorageOptions = ..., partition_cols: list[str] | None = ..., **kwargs) -> None:
        ...
    
    def read(self, path, columns=..., use_nullable_dtypes=..., storage_options: StorageOptions = ..., **kwargs) -> DataFrame:
        ...
    


class FastParquetImpl(BaseImpl):
    def __init__(self) -> None:
        ...
    
    def write(self, df: DataFrame, path, compression=..., index=..., partition_cols=..., storage_options: StorageOptions = ..., **kwargs) -> None:
        ...
    
    def read(self, path, columns=..., storage_options: StorageOptions = ..., **kwargs) -> DataFrame:
        ...
    


@doc(storage_options=_shared_docs["storage_options"])
def to_parquet(df: DataFrame, path: FilePath | WriteBuffer[bytes] | None = ..., engine: str = ..., compression: str | None = ..., index: bool | None = ..., storage_options: StorageOptions = ..., partition_cols: list[str] | None = ..., **kwargs) -> bytes | None:
    """
    Write a DataFrame to the parquet format.

    Parameters
    ----------
    df : DataFrame
    path : str, path object, file-like object, or None, default None
        String, path object (implementing ``os.PathLike[str]``), or file-like
        object implementing a binary ``write()`` function. If None, the result is
        returned as bytes. If a string, it will be used as Root Directory path
        when writing a partitioned dataset. The engine fastparquet does not
        accept file-like objects.

        .. versionchanged:: 1.2.0

    engine : {{'auto', 'pyarrow', 'fastparquet'}}, default 'auto'
        Parquet library to use. If 'auto', then the option
        ``io.parquet.engine`` is used. The default ``io.parquet.engine``
        behavior is to try 'pyarrow', falling back to 'fastparquet' if
        'pyarrow' is unavailable.
    compression : {{'snappy', 'gzip', 'brotli', 'lz4', 'zstd', None}},
        default 'snappy'. Name of the compression to use. Use ``None``
        for no compression. The supported compression methods actually
        depend on which engine is used. For 'pyarrow', 'snappy', 'gzip',
        'brotli', 'lz4', 'zstd' are all supported. For 'fastparquet',
        only 'gzip' and 'snappy' are supported.
    index : bool, default None
        If ``True``, include the dataframe's index(es) in the file output. If
        ``False``, they will not be written to the file.
        If ``None``, similar to ``True`` the dataframe's index(es)
        will be saved. However, instead of being saved as values,
        the RangeIndex will be stored as a range in the metadata so it
        doesn't require much space and is faster. Other indexes will
        be included as columns in the file output.
    partition_cols : str or list, optional, default None
        Column names by which to partition the dataset.
        Columns are partitioned in the order they are given.
        Must be None if path is not a string.
    {storage_options}

        .. versionadded:: 1.2.0

    kwargs
        Additional keyword arguments passed to the engine

    Returns
    -------
    bytes if no path argument is provided else None
    """
    ...

@doc(storage_options=_shared_docs["storage_options"])
def read_parquet(path: FilePath | ReadBuffer[bytes], engine: str = ..., columns: list[str] | None = ..., storage_options: StorageOptions = ..., use_nullable_dtypes: bool = ..., **kwargs) -> DataFrame:
    """
    Load a parquet object from the file path, returning a DataFrame.

    Parameters
    ----------
    path : str, path object or file-like object
        String, path object (implementing ``os.PathLike[str]``), or file-like
        object implementing a binary ``read()`` function.
        The string could be a URL. Valid URL schemes include http, ftp, s3,
        gs, and file. For file URLs, a host is expected. A local file could be:
        ``file://localhost/path/to/table.parquet``.
        A file URL can also be a path to a directory that contains multiple
        partitioned parquet files. Both pyarrow and fastparquet support
        paths to directories as well as file URLs. A directory path could be:
        ``file://localhost/path/to/tables`` or ``s3://bucket/partition_dir``.
    engine : {{'auto', 'pyarrow', 'fastparquet'}}, default 'auto'
        Parquet library to use. If 'auto', then the option
        ``io.parquet.engine`` is used. The default ``io.parquet.engine``
        behavior is to try 'pyarrow', falling back to 'fastparquet' if
        'pyarrow' is unavailable.
    columns : list, default=None
        If not None, only these columns will be read from the file.

    {storage_options}

        .. versionadded:: 1.3.0

    use_nullable_dtypes : bool, default False
        If True, use dtypes that use ``pd.NA`` as missing value indicator
        for the resulting DataFrame. (only applicable for the ``pyarrow``
        engine)
        As new dtypes are added that support ``pd.NA`` in the future, the
        output with this option will change to use those dtypes.
        Note: this is an experimental option, and behaviour (e.g. additional
        support dtypes) may change without notice.

        .. versionadded:: 1.2.0

    **kwargs
        Any additional kwargs are passed to the engine.

    Returns
    -------
    DataFrame
    """
    ...




"""
This type stub file was generated by pyright.
"""

import textwrap
import numpy as np
from typing import Literal, Sequence, TYPE_CHECKING, TypeVar, Union, overload
from pandas._libs.interval import Interval, IntervalMixin
from pandas._typing import Dtype, IntervalClosedType, NpDtype, PositionalIndexer, ScalarIndexer, SequenceIndexer, npt
from pandas.util._decorators import Appender, deprecate_nonkeyword_arguments
from pandas.core.dtypes.dtypes import IntervalDtype
from pandas.core.arrays.base import ExtensionArray, _extension_array_shared_docs
from pandas.core.ops import unpack_zerodim_and_defer
from pandas import Index, Series

if TYPE_CHECKING:
    ...
IntervalArrayT = TypeVar("IntervalArrayT", bound="IntervalArray")
IntervalOrNA = Union[Interval, float]
_interval_shared_docs: dict[str, str] = ...
_shared_docs_kwargs = ...
@Appender(_interval_shared_docs["class"] % { "klass": "IntervalArray","summary": "Pandas array for interval data that are closed on the same side.","versionadded": "0.24.0","name": "","extra_attributes": "","extra_methods": "","examples": textwrap.dedent("""\
    Examples
    --------
    A new ``IntervalArray`` can be constructed directly from an array-like of
    ``Interval`` objects:

    >>> pd.arrays.IntervalArray([pd.Interval(0, 1), pd.Interval(1, 5)])
    <IntervalArray>
    [(0, 1], (1, 5]]
    Length: 2, dtype: interval[int64, right]

    It may also be constructed using one of the constructor
    methods: :meth:`IntervalArray.from_arrays`,
    :meth:`IntervalArray.from_breaks`, and :meth:`IntervalArray.from_tuples`.
    """) })
class IntervalArray(IntervalMixin, ExtensionArray):
    can_hold_na = ...
    _na_value = ...
    @property
    def ndim(self) -> Literal[1]:
        ...
    
    _left: np.ndarray
    _right: np.ndarray
    _dtype: IntervalDtype
    def __new__(cls: type[IntervalArrayT], data, closed=..., dtype: Dtype | None = ..., copy: bool = ..., verify_integrity: bool = ...): # -> IntervalArrayT@__new__:
        ...
    
    @classmethod
    @Appender(_interval_shared_docs["from_breaks"] % { "klass": "IntervalArray","examples": textwrap.dedent("""\
        Examples
        --------
        >>> pd.arrays.IntervalArray.from_breaks([0, 1, 2, 3])
        <IntervalArray>
        [(0, 1], (1, 2], (2, 3]]
        Length: 3, dtype: interval[int64, right]
        """) })
    def from_breaks(cls: type[IntervalArrayT], breaks, closed: IntervalClosedType | None = ..., copy: bool = ..., dtype: Dtype | None = ...) -> IntervalArrayT:
        ...
    
    @classmethod
    @Appender(_interval_shared_docs["from_arrays"] % { "klass": "IntervalArray","examples": textwrap.dedent("""\
        >>> pd.arrays.IntervalArray.from_arrays([0, 1, 2], [1, 2, 3])
        <IntervalArray>
        [(0, 1], (1, 2], (2, 3]]
        Length: 3, dtype: interval[int64, right]
        """) })
    def from_arrays(cls: type[IntervalArrayT], left, right, closed: IntervalClosedType | None = ..., copy: bool = ..., dtype: Dtype | None = ...) -> IntervalArrayT:
        ...
    
    @classmethod
    @Appender(_interval_shared_docs["from_tuples"] % { "klass": "IntervalArray","examples": textwrap.dedent("""\
        Examples
        --------
        >>> pd.arrays.IntervalArray.from_tuples([(0, 1), (1, 2)])
        <IntervalArray>
        [(0, 1], (1, 2]]
        Length: 2, dtype: interval[int64, right]
        """) })
    def from_tuples(cls: type[IntervalArrayT], data, closed=..., copy: bool = ..., dtype: Dtype | None = ...) -> IntervalArrayT:
        ...
    
    @property
    def dtype(self) -> IntervalDtype:
        ...
    
    @property
    def nbytes(self) -> int:
        ...
    
    @property
    def size(self) -> int:
        ...
    
    def __iter__(self): # -> Any:
        ...
    
    def __len__(self) -> int:
        ...
    
    @overload
    def __getitem__(self, key: ScalarIndexer) -> IntervalOrNA:
        ...
    
    @overload
    def __getitem__(self: IntervalArrayT, key: SequenceIndexer) -> IntervalArrayT:
        ...
    
    def __getitem__(self: IntervalArrayT, key: PositionalIndexer) -> IntervalArrayT | IntervalOrNA:
        ...
    
    def __setitem__(self, key, value) -> None:
        ...
    
    @unpack_zerodim_and_defer("__eq__")
    def __eq__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__ne__")
    def __ne__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__gt__")
    def __gt__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__ge__")
    def __ge__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__lt__")
    def __lt__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__le__")
    def __le__(self, other) -> bool:
        ...
    
    @deprecate_nonkeyword_arguments(version=None, allowed_args=["self"])
    def argsort(self, ascending: bool = ..., kind: str = ..., na_position: str = ..., *args, **kwargs) -> np.ndarray:
        ...
    
    def min(self, *, axis: int | None = ..., skipna: bool = ...) -> IntervalOrNA:
        ...
    
    def max(self, *, axis: int | None = ..., skipna: bool = ...) -> IntervalOrNA:
        ...
    
    def fillna(self: IntervalArrayT, value=..., method=..., limit=...) -> IntervalArrayT:
        """
        Fill NA/NaN values using the specified method.

        Parameters
        ----------
        value : scalar, dict, Series
            If a scalar value is passed it is used to fill all missing values.
            Alternatively, a Series or dict can be used to fill in different
            values for each index. The value should not be a list. The
            value(s) passed should be either Interval objects or NA/NaN.
        method : {'backfill', 'bfill', 'pad', 'ffill', None}, default None
            (Not implemented yet for IntervalArray)
            Method to use for filling holes in reindexed Series
        limit : int, default None
            (Not implemented yet for IntervalArray)
            If method is specified, this is the maximum number of consecutive
            NaN values to forward/backward fill. In other words, if there is
            a gap with more than this number of consecutive NaNs, it will only
            be partially filled. If method is not specified, this is the
            maximum number of entries along the entire axis where NaNs will be
            filled.

        Returns
        -------
        filled : IntervalArray with NA/NaN filled
        """
        ...
    
    def astype(self, dtype, copy: bool = ...): # -> ArrayLike | Self@IntervalArray:
        """
        Cast to an ExtensionArray or NumPy array with dtype 'dtype'.

        Parameters
        ----------
        dtype : str or dtype
            Typecode or data-type to which the array is cast.

        copy : bool, default True
            Whether to copy the data, even if not necessary. If False,
            a copy is made only if the old dtype does not match the
            new dtype.

        Returns
        -------
        array : ExtensionArray or ndarray
            ExtensionArray or NumPy ndarray with 'dtype' for its dtype.
        """
        ...
    
    def equals(self, other) -> bool:
        ...
    
    def copy(self: IntervalArrayT) -> IntervalArrayT:
        """
        Return a copy of the array.

        Returns
        -------
        IntervalArray
        """
        ...
    
    def isna(self) -> np.ndarray:
        ...
    
    def shift(self, periods: int = ..., fill_value: object = ...) -> IntervalArray:
        ...
    
    def take(self: IntervalArrayT, indices, *, allow_fill: bool = ..., fill_value=..., axis=..., **kwargs) -> IntervalArrayT:
        """
        Take elements from the IntervalArray.

        Parameters
        ----------
        indices : sequence of integers
            Indices to be taken.

        allow_fill : bool, default False
            How to handle negative values in `indices`.

            * False: negative values in `indices` indicate positional indices
              from the right (the default). This is similar to
              :func:`numpy.take`.

            * True: negative values in `indices` indicate
              missing values. These values are set to `fill_value`. Any other
              other negative values raise a ``ValueError``.

        fill_value : Interval or NA, optional
            Fill value to use for NA-indices when `allow_fill` is True.
            This may be ``None``, in which case the default NA value for
            the type, ``self.dtype.na_value``, is used.

            For many ExtensionArrays, there will be two representations of
            `fill_value`: a user-facing "boxed" scalar, and a low-level
            physical NA value. `fill_value` should be the user-facing version,
            and the implementation should handle translating that to the
            physical version for processing the take if necessary.

        axis : any, default None
            Present for compat with IntervalIndex; does nothing.

        Returns
        -------
        IntervalArray

        Raises
        ------
        IndexError
            When the indices are out of bounds for the array.
        ValueError
            When `indices` contains negative values other than ``-1``
            and `allow_fill` is True.
        """
        ...
    
    def value_counts(self, dropna: bool = ...) -> Series:
        """
        Returns a Series containing counts of each interval.

        Parameters
        ----------
        dropna : bool, default True
            Don't include counts of NaN.

        Returns
        -------
        counts : Series

        See Also
        --------
        Series.value_counts
        """
        ...
    
    def __repr__(self) -> str:
        ...
    
    @property
    def left(self): # -> Index:
        """
        Return the left endpoints of each Interval in the IntervalArray as an Index.
        """
        ...
    
    @property
    def right(self): # -> Index:
        """
        Return the right endpoints of each Interval in the IntervalArray as an Index.
        """
        ...
    
    @property
    def length(self) -> Index:
        """
        Return an Index with entries denoting the length of each Interval.
        """
        ...
    
    @property
    def mid(self) -> Index:
        """
        Return the midpoint of each Interval in the IntervalArray as an Index.
        """
        ...
    
    @Appender(_interval_shared_docs["overlaps"] % { "klass": "IntervalArray","examples": textwrap.dedent("""\
        >>> data = [(0, 1), (1, 3), (2, 4)]
        >>> intervals = pd.arrays.IntervalArray.from_tuples(data)
        >>> intervals
        <IntervalArray>
        [(0, 1], (1, 3], (2, 4]]
        Length: 3, dtype: interval[int64, right]
        """) })
    def overlaps(self, other): # -> Any:
        ...
    
    @property
    def closed(self) -> IntervalClosedType:
        """
        String describing the inclusive side the intervals.

        Either ``left``, ``right``, ``both`` or ``neither``.
        """
        ...
    
    @Appender(_interval_shared_docs["set_closed"] % { "klass": "IntervalArray","examples": textwrap.dedent("""\
        Examples
        --------
        >>> index = pd.arrays.IntervalArray.from_breaks(range(4))
        >>> index
        <IntervalArray>
        [(0, 1], (1, 2], (2, 3]]
        Length: 3, dtype: interval[int64, right]
        >>> index.set_closed('both')
        <IntervalArray>
        [[0, 1], [1, 2], [2, 3]]
        Length: 3, dtype: interval[int64, both]
        """) })
    def set_closed(self: IntervalArrayT, closed: IntervalClosedType) -> IntervalArrayT:
        ...
    
    @property
    @Appender(_interval_shared_docs["is_non_overlapping_monotonic"] % _shared_docs_kwargs)
    def is_non_overlapping_monotonic(self) -> bool:
        ...
    
    def __array__(self, dtype: NpDtype | None = ...) -> np.ndarray:
        """
        Return the IntervalArray's data as a numpy array of Interval
        objects (with dtype='object')
        """
        ...
    
    def __arrow_array__(self, type=...):
        """
        Convert myself into a pyarrow Array.
        """
        ...
    
    @Appender(_interval_shared_docs["to_tuples"] % { "return_type": "ndarray","examples": "" })
    def to_tuples(self, na_tuple=...) -> np.ndarray:
        ...
    
    def insert(self: IntervalArrayT, loc: int, item: Interval) -> IntervalArrayT:
        """
        Return a new IntervalArray inserting new item at location. Follows
        Python numpy.insert semantics for negative values.  Only Interval
        objects and NA can be inserted into an IntervalIndex

        Parameters
        ----------
        loc : int
        item : Interval

        Returns
        -------
        IntervalArray
        """
        ...
    
    def delete(self: IntervalArrayT, loc) -> IntervalArrayT:
        ...
    
    @Appender(_extension_array_shared_docs["repeat"] % _shared_docs_kwargs)
    def repeat(self: IntervalArrayT, repeats: int | Sequence[int], axis: int | None = ...) -> IntervalArrayT:
        ...
    
    @Appender(_interval_shared_docs["contains"] % { "klass": "IntervalArray","examples": textwrap.dedent("""\
        >>> intervals = pd.arrays.IntervalArray.from_tuples([(0, 1), (1, 3), (2, 4)])
        >>> intervals
        <IntervalArray>
        [(0, 1], (1, 3], (2, 4]]
        Length: 3, dtype: interval[int64, right]
        """) })
    def contains(self, other):
        ...
    
    def isin(self, values) -> npt.NDArray[np.bool_]:
        ...
    
    def unique(self) -> IntervalArray:
        ...
    





"""
This type stub file was generated by pyright.
"""

import re
import numpy as np
from typing import Any, Pattern
from pandas._typing import ArrayLike, Scalar, npt

"""
Methods used by Block.replace and related methods.
"""
def should_use_regex(regex: bool, to_replace: Any) -> bool:
    """
    Decide whether to treat `to_replace` as a regular expression.
    """
    ...

def compare_or_regex_search(a: ArrayLike, b: Scalar | Pattern, regex: bool, mask: npt.NDArray[np.bool_]) -> ArrayLike | bool:
    """
    Compare two array-like inputs of the same shape or two scalar values

    Calls operator.eq or re.search, depending on regex argument. If regex is
    True, perform an element-wise regex matching.

    Parameters
    ----------
    a : array-like
    b : scalar or regex pattern
    regex : bool
    mask : np.ndarray[bool]

    Returns
    -------
    mask : array-like of bool
    """
    ...

def replace_regex(values: ArrayLike, rx: re.Pattern, value, mask: npt.NDArray[np.bool_] | None) -> None:
    """
    Parameters
    ----------
    values : ArrayLike
        Object dtype.
    rx : re.Pattern
    value : Any
    mask : np.ndarray[bool], optional

    Notes
    -----
    Alters values in-place.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import TYPE_CHECKING
from pandas._libs.tslibs import Tick
from pandas._typing import NpDtype, npt
from pandas.core.arrays import datetimelike as dtl
from pandas.core.ops.common import unpack_zerodim_and_defer
from pandas import DataFrame

if TYPE_CHECKING:
    ...
class TimedeltaArray(dtl.TimelikeOps):
    """
    Pandas ExtensionArray for timedelta data.

    .. warning::

       TimedeltaArray is currently experimental, and its API may change
       without warning. In particular, :attr:`TimedeltaArray.dtype` is
       expected to change to be an instance of an ``ExtensionDtype``
       subclass.

    Parameters
    ----------
    values : array-like
        The timedelta data.

    dtype : numpy.dtype
        Currently, only ``numpy.dtype("timedelta64[ns]")`` is accepted.
    freq : Offset, optional
    copy : bool, default False
        Whether to copy the underlying array of data.

    Attributes
    ----------
    None

    Methods
    -------
    None
    """
    _typ = ...
    _internal_fill_value = ...
    _recognized_scalars = ...
    _is_recognized_dtype = ...
    _infer_matches = ...
    __array_priority__ = ...
    _other_ops: list[str] = ...
    _bool_ops: list[str] = ...
    _object_ops: list[str] = ...
    _field_ops: list[str] = ...
    _datetimelike_ops: list[str] = ...
    _datetimelike_methods: list[str] = ...
    @property
    def dtype(self) -> np.dtype:
        """
        The dtype for the TimedeltaArray.

        .. warning::

           A future version of pandas will change dtype to be an instance
           of a :class:`pandas.api.extensions.ExtensionDtype` subclass,
           not a ``numpy.dtype``.

        Returns
        -------
        numpy.dtype
        """
        ...
    
    _freq = ...
    _default_dtype = ...
    def astype(self, dtype, copy: bool = ...): # -> ndarray[Unknown, Unknown] | NDArray[object_] | ndarray[Any, Unknown] | ExtensionArray | NDArray[Any] | NDArray[int64] | NDArray[Unknown]:
        ...
    
    def __iter__(self): # -> Generator[DTScalarOrNaT | Any, None, None]:
        ...
    
    def sum(self, *, axis: int | None = ..., dtype: NpDtype | None = ..., out=..., keepdims: bool = ..., initial=..., skipna: bool = ..., min_count: int = ...):
        ...
    
    def std(self, *, axis: int | None = ..., dtype: NpDtype | None = ..., out=..., ddof: int = ..., keepdims: bool = ..., skipna: bool = ...): # -> Timedelta | NaTType:
        ...
    
    @unpack_zerodim_and_defer("__mul__")
    def __mul__(self, other) -> TimedeltaArray:
        ...
    
    __rmul__ = ...
    @unpack_zerodim_and_defer("__truediv__")
    def __truediv__(self, other): # -> NDArray[float64] | Any | TimedeltaArray | NDArray[Any] | ndarray[Any, dtype[Unknown]]:
        ...
    
    @unpack_zerodim_and_defer("__rtruediv__")
    def __rtruediv__(self, other): # -> NDArray[float64] | Any | NDArray[Any | Unknown]:
        ...
    
    @unpack_zerodim_and_defer("__floordiv__")
    def __floordiv__(self, other): # -> NDArray[float64] | NDArray[timedelta64] | Self@TimedeltaArray | NDArray[signedinteger[Any]] | TimedeltaArray | ndarray[Any, dtype[Unknown]]:
        ...
    
    @unpack_zerodim_and_defer("__rfloordiv__")
    def __rfloordiv__(self, other): # -> NDArray[float64] | NDArray[intp] | NDArray[Any | Unknown]:
        ...
    
    @unpack_zerodim_and_defer("__mod__")
    def __mod__(self, other):
        ...
    
    @unpack_zerodim_and_defer("__rmod__")
    def __rmod__(self, other):
        ...
    
    @unpack_zerodim_and_defer("__divmod__")
    def __divmod__(self, other): # -> tuple[NDArray[float64] | NDArray[timedelta64] | Unknown | Self@TimedeltaArray | NDArray[signedinteger[Any]] | TimedeltaArray | ndarray[Any, dtype[Unknown]], Unknown]:
        ...
    
    @unpack_zerodim_and_defer("__rdivmod__")
    def __rdivmod__(self, other): # -> tuple[NDArray[float64] | NDArray[intp] | Unknown | NDArray[Any | Unknown], Unknown]:
        ...
    
    def __neg__(self) -> TimedeltaArray:
        ...
    
    def __pos__(self) -> TimedeltaArray:
        ...
    
    def __abs__(self) -> TimedeltaArray:
        ...
    
    def total_seconds(self) -> npt.NDArray[np.float64]:
        """
        Return total duration of each element expressed in seconds.

        This method is available directly on TimedeltaArray, TimedeltaIndex
        and on Series containing timedelta values under the ``.dt`` namespace.

        Returns
        -------
        seconds : [ndarray, Float64Index, Series]
            When the calling object is a TimedeltaArray, the return type
            is ndarray.  When the calling object is a TimedeltaIndex,
            the return type is a Float64Index. When the calling object
            is a Series, the return type is Series of type `float64` whose
            index is the same as the original.

        See Also
        --------
        datetime.timedelta.total_seconds : Standard library version
            of this method.
        TimedeltaIndex.components : Return a DataFrame with components of
            each Timedelta.

        Examples
        --------
        **Series**

        >>> s = pd.Series(pd.to_timedelta(np.arange(5), unit='d'))
        >>> s
        0   0 days
        1   1 days
        2   2 days
        3   3 days
        4   4 days
        dtype: timedelta64[ns]

        >>> s.dt.total_seconds()
        0         0.0
        1     86400.0
        2    172800.0
        3    259200.0
        4    345600.0
        dtype: float64

        **TimedeltaIndex**

        >>> idx = pd.to_timedelta(np.arange(5), unit='d')
        >>> idx
        TimedeltaIndex(['0 days', '1 days', '2 days', '3 days', '4 days'],
                       dtype='timedelta64[ns]', freq=None)

        >>> idx.total_seconds()
        Float64Index([0.0, 86400.0, 172800.0, 259200.0, 345600.0],
                     dtype='float64')
        """
        ...
    
    def to_pytimedelta(self) -> npt.NDArray[np.object_]:
        """
        Return an ndarray of datetime.timedelta objects.

        Returns
        -------
        timedeltas : ndarray[object]
        """
        ...
    
    days = ...
    seconds = ...
    microseconds = ...
    nanoseconds = ...
    @property
    def components(self) -> DataFrame:
        """
        Return a DataFrame of the individual resolution components of the Timedeltas.

        The components (days, hours, minutes seconds, milliseconds, microseconds,
        nanoseconds) are returned as columns in a DataFrame.

        Returns
        -------
        DataFrame
        """
        ...
    


def sequence_to_td64ns(data, copy: bool = ..., unit=..., errors=...) -> tuple[np.ndarray, Tick | None]:
    """
    Parameters
    ----------
    data : list-like
    copy : bool, default False
    unit : str, optional
        The timedelta unit to treat integers as multiples of. For numeric
        data this defaults to ``'ns'``.
        Must be un-specified if the data contains a str and ``errors=="raise"``.
    errors : {"raise", "coerce", "ignore"}, default "raise"
        How to handle elements that cannot be converted to timedelta64[ns].
        See ``pandas.to_timedelta`` for details.

    Returns
    -------
    converted : numpy.ndarray
        The sequence converted to a numpy array with dtype ``timedelta64[ns]``.
    inferred_freq : Tick or None
        The inferred frequency of the sequence.

    Raises
    ------
    ValueError : Data cannot be converted to timedelta64[ns].

    Notes
    -----
    Unlike `pandas.to_timedelta`, if setting ``errors=ignore`` will not cause
    errors to be ignored; they are caught and subsequently ignored at a
    higher level.
    """
    ...

def ints_to_td64ns(data, unit=...): # -> tuple[ndarray[Unknown, Unknown] | Unknown, bool]:
    """
    Convert an ndarray with integer-dtype to timedelta64[ns] dtype, treating
    the integers as multiples of the given timedelta unit.

    Parameters
    ----------
    data : numpy.ndarray with integer-dtype
    unit : str, default "ns"
        The timedelta unit to treat integers as multiples of.

    Returns
    -------
    numpy.ndarray : timedelta64[ns] array converted from data
    bool : whether a copy was made
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numba
import numpy as np

"""
Numba 1D min/max kernels that can be shared by
* Dataframe / Series
* groupby
* rolling / expanding

Mirrors pandas/_libs/window/aggregation.pyx
"""
@numba.jit(nopython=True, nogil=True, parallel=False)
def sliding_min_max(values: np.ndarray, start: np.ndarray, end: np.ndarray, min_periods: int, is_max: bool) -> np.ndarray:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._libs.tslibs import Timestamp
from pandas._libs.tslibs.offsets import BaseOffset
from pandas._typing import npt
from pandas.util._decorators import cache_readonly

_offset_to_period_map = ...
_need_suffix = ...
def get_period_alias(offset_str: str) -> str | None:
    """
    Alias to closest period strings BQ->Q etc.
    """
    ...

def get_offset(name: str) -> BaseOffset:
    """
    Return DateOffset object associated with rule name.

    .. deprecated:: 1.0.0

    Examples
    --------
    get_offset('EOM') --> BMonthEnd(1)
    """
    ...

def infer_freq(index, warn: bool = ...) -> str | None:
    """
    Infer the most likely frequency given the input index.

    Parameters
    ----------
    index : DatetimeIndex or TimedeltaIndex
      If passed a Series will use the values of the series (NOT THE INDEX).
    warn : bool, default True
      .. deprecated:: 1.5.0

    Returns
    -------
    str or None
        None if no discernible frequency.

    Raises
    ------
    TypeError
        If the index is not datetime-like.
    ValueError
        If there are fewer than three values.

    Examples
    --------
    >>> idx = pd.date_range(start='2020/12/01', end='2020/12/30', periods=30)
    >>> pd.infer_freq(idx)
    'D'
    """
    ...

class _FrequencyInferer:
    """
    Not sure if I can avoid the state machine here
    """
    def __init__(self, index, warn: bool = ...) -> None:
        ...
    
    @cache_readonly
    def deltas(self) -> npt.NDArray[np.int64]:
        ...
    
    @cache_readonly
    def deltas_asi8(self) -> npt.NDArray[np.int64]:
        ...
    
    @cache_readonly
    def is_unique(self) -> bool:
        ...
    
    @cache_readonly
    def is_unique_asi8(self) -> bool:
        ...
    
    def get_freq(self) -> str | None:
        """
        Find the appropriate frequency string to describe the inferred
        frequency of self.i8values

        Returns
        -------
        str or None
        """
        ...
    
    @cache_readonly
    def day_deltas(self) -> list[int]:
        ...
    
    @cache_readonly
    def hour_deltas(self) -> list[int]:
        ...
    
    @cache_readonly
    def fields(self) -> np.ndarray:
        ...
    
    @cache_readonly
    def rep_stamp(self) -> Timestamp:
        ...
    
    def month_position_check(self) -> str | None:
        ...
    
    @cache_readonly
    def mdiffs(self) -> npt.NDArray[np.int64]:
        ...
    
    @cache_readonly
    def ydiffs(self) -> npt.NDArray[np.int64]:
        ...
    


class _TimedeltaFrequencyInferer(_FrequencyInferer):
    ...


def is_subperiod(source, target) -> bool:
    """
    Returns True if downsampling is possible between source and target
    frequencies

    Parameters
    ----------
    source : str or DateOffset
        Frequency converting from
    target : str or DateOffset
        Frequency converting to

    Returns
    -------
    bool
    """
    ...

def is_superperiod(source, target) -> bool:
    """
    Returns True if upsampling is possible between source and target
    frequencies

    Parameters
    ----------
    source : str or DateOffset
        Frequency converting from
    target : str or DateOffset
        Frequency converting to

    Returns
    -------
    bool
    """
    ...

__all__ = ["Day", "get_offset", "get_period_alias", "infer_freq", "is_subperiod", "is_superperiod", "to_offset"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Hashable, Literal
from pandas._typing import ArrayLike, Dtype, npt

STR_NA_VALUES: set[str]
def sanitize_objects(values: npt.NDArray[np.object_], na_values: set) -> int:
    ...

class TextReader:
    unnamed_cols: set[str]
    table_width: int
    leading_cols: int
    header: list[list[int]]
    def __init__(self, source, delimiter: bytes | str = ..., header=..., header_start: int = ..., header_end: int = ..., index_col=..., names=..., tokenize_chunksize: int = ..., delim_whitespace: bool = ..., converters=..., skipinitialspace: bool = ..., escapechar: bytes | str | None = ..., doublequote: bool = ..., quotechar: str | bytes | None = ..., quoting: int = ..., lineterminator: bytes | str | None = ..., comment=..., decimal: bytes | str = ..., thousands: bytes | str | None = ..., dtype: Dtype | dict[Hashable, Dtype] = ..., usecols=..., error_bad_lines: bool = ..., warn_bad_lines: bool = ..., na_filter: bool = ..., na_values=..., na_fvalues=..., keep_default_na: bool = ..., true_values=..., false_values=..., allow_leading_cols: bool = ..., skiprows=..., skipfooter: int = ..., verbose: bool = ..., mangle_dupe_cols: bool = ..., float_precision: Literal["round_trip", "legacy", "high"] | None = ..., skip_blank_lines: bool = ..., encoding_errors: bytes | str = ...) -> None:
        ...
    
    def set_noconvert(self, i: int) -> None:
        ...
    
    def remove_noconvert(self, i: int) -> None:
        ...
    
    def close(self) -> None:
        ...
    
    def read(self, rows: int | None = ...) -> dict[int, ArrayLike]:
        ...
    
    def read_low_memory(self, rows: int | None) -> list[dict[int, ArrayLike]]:
        ...
    





"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

"""
masked_reductions.py is for reduction algorithms using a mask-based approach
for missing values.
"""
def sum(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., min_count: int = ..., axis: int | None = ...): # -> NAType:
    ...

def prod(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., min_count: int = ..., axis: int | None = ...): # -> NAType:
    ...

def min(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., axis: int | None = ...): # -> NAType:
    ...

def max(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., axis: int | None = ...): # -> NAType:
    ...

def mean(values: np.ndarray, mask: npt.NDArray[np.bool_], skipna: bool = ...): # -> NAType:
    ...




"""
This type stub file was generated by pyright.
"""

"""
Data IO api
"""
__all__ = ["ExcelFile", "ExcelWriter", "HDFStore", "read_clipboard", "read_csv", "read_excel", "read_feather", "read_fwf", "read_gbq", "read_hdf", "read_html", "read_json", "read_orc", "read_parquet", "read_pickle", "read_sas", "read_spss", "read_sql", "read_sql_query", "read_sql_table", "read_stata", "read_table", "read_xml", "to_pickle"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Callable
from pandas._typing import ArrayLike, Dtype, F, npt
from pandas.core.dtypes.dtypes import PeriodDtype

bn = ...
_BOTTLENECK_INSTALLED = ...
_USE_BOTTLENECK = ...
def set_use_bottleneck(v: bool = ...) -> None:
    ...

class disallow:
    def __init__(self, *dtypes: Dtype) -> None:
        ...
    
    def check(self, obj) -> bool:
        ...
    
    def __call__(self, f: F) -> F:
        ...
    


class bottleneck_switch:
    def __init__(self, name=..., **kwargs) -> None:
        ...
    
    def __call__(self, alt: F) -> F:
        ...
    


def maybe_operate_rowwise(func: F) -> F:
    """
    NumPy operations on C-contiguous ndarrays with axis=1 can be
    very slow if axis 1 >> axis 0.
    Operate row-by-row and concatenate the results.
    """
    ...

def nanany(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> bool:
    """
    Check if any elements along an axis evaluate to True.

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : bool

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, 2])
    >>> nanops.nanany(s)
    True

    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([np.nan])
    >>> nanops.nanany(s)
    False
    """
    ...

def nanall(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> bool:
    """
    Check if all elements along an axis evaluate to True.

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : bool

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, 2, np.nan])
    >>> nanops.nanall(s)
    True

    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, 0])
    >>> nanops.nanall(s)
    False
    """
    ...

@disallow("M8")
@_datetimelike_compat
@maybe_operate_rowwise
def nansum(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., min_count: int = ..., mask: npt.NDArray[np.bool_] | None = ...) -> float:
    """
    Sum the elements along an axis ignoring NaNs

    Parameters
    ----------
    values : ndarray[dtype]
    axis : int, optional
    skipna : bool, default True
    min_count: int, default 0
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : dtype

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, 2, np.nan])
    >>> nanops.nansum(s)
    3.0
    """
    ...

@disallow(PeriodDtype)
@bottleneck_switch()
@_datetimelike_compat
def nanmean(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> float:
    """
    Compute the mean of the element along an axis ignoring NaNs

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    float
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, 2, np.nan])
    >>> nanops.nanmean(s)
    1.5
    """
    ...

@bottleneck_switch()
def nanmedian(values, *, axis=..., skipna=..., mask=...): # -> Any | NDArray[Unknown]:
    """
    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : float
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, np.nan, 2, 2])
    >>> nanops.nanmedian(s)
    2.0
    """
    ...

def get_empty_reduction_result(shape: tuple[int, ...], axis: int, dtype: np.dtype | type[np.floating], fill_value: Any) -> np.ndarray:
    """
    The result from a reduction on an empty ndarray.

    Parameters
    ----------
    shape : Tuple[int]
    axis : int
    dtype : np.dtype
    fill_value : Any

    Returns
    -------
    np.ndarray
    """
    ...

@bottleneck_switch(ddof=1)
def nanstd(values, *, axis=..., skipna=..., ddof=..., mask=...): # -> Any | NDArray[Unknown]:
    """
    Compute the standard deviation along given axis while ignoring NaNs

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    ddof : int, default 1
        Delta Degrees of Freedom. The divisor used in calculations is N - ddof,
        where N represents the number of elements.
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : float
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, np.nan, 2, 3])
    >>> nanops.nanstd(s)
    1.0
    """
    ...

@disallow("M8", "m8")
@bottleneck_switch(ddof=1)
def nanvar(values, *, axis=..., skipna=..., ddof=..., mask=...): # -> Any:
    """
    Compute the variance along given axis while ignoring NaNs

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    ddof : int, default 1
        Delta Degrees of Freedom. The divisor used in calculations is N - ddof,
        where N represents the number of elements.
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : float
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, np.nan, 2, 3])
    >>> nanops.nanvar(s)
    1.0
    """
    ...

@disallow("M8", "m8")
def nansem(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., ddof: int = ..., mask: npt.NDArray[np.bool_] | None = ...) -> float:
    """
    Compute the standard error in the mean along given axis while ignoring NaNs

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    ddof : int, default 1
        Delta Degrees of Freedom. The divisor used in calculations is N - ddof,
        where N represents the number of elements.
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : float64
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, np.nan, 2, 3])
    >>> nanops.nansem(s)
     0.5773502691896258
    """
    ...

nanmin = ...
nanmax = ...
@disallow("O")
def nanargmax(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> int | np.ndarray:
    """
    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : int or ndarray[int]
        The index/indices  of max value in specified axis or -1 in the NA case

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> arr = np.array([1, 2, 3, np.nan, 4])
    >>> nanops.nanargmax(arr)
    4

    >>> arr = np.array(range(12), dtype=np.float64).reshape(4, 3)
    >>> arr[2:, 2] = np.nan
    >>> arr
    array([[ 0.,  1.,  2.],
           [ 3.,  4.,  5.],
           [ 6.,  7., nan],
           [ 9., 10., nan]])
    >>> nanops.nanargmax(arr, axis=1)
    array([2, 2, 1, 1])
    """
    ...

@disallow("O")
def nanargmin(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> int | np.ndarray:
    """
    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : int or ndarray[int]
        The index/indices of min value in specified axis or -1 in the NA case

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> arr = np.array([1, 2, 3, np.nan, 4])
    >>> nanops.nanargmin(arr)
    0

    >>> arr = np.array(range(12), dtype=np.float64).reshape(4, 3)
    >>> arr[2:, 0] = np.nan
    >>> arr
    array([[ 0.,  1.,  2.],
           [ 3.,  4.,  5.],
           [nan,  7.,  8.],
           [nan, 10., 11.]])
    >>> nanops.nanargmin(arr, axis=1)
    array([0, 0, 1, 1])
    """
    ...

@disallow("M8", "m8")
@maybe_operate_rowwise
def nanskew(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> float:
    """
    Compute the sample skewness.

    The statistic computed here is the adjusted Fisher-Pearson standardized
    moment coefficient G1. The algorithm computes this coefficient directly
    from the second and third central moment.

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : float64
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, np.nan, 1, 2])
    >>> nanops.nanskew(s)
    1.7320508075688787
    """
    ...

@disallow("M8", "m8")
@maybe_operate_rowwise
def nankurt(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> float:
    """
    Compute the sample excess kurtosis

    The statistic computed here is the adjusted Fisher-Pearson standardized
    moment coefficient G2, computed directly from the second and fourth
    central moment.

    Parameters
    ----------
    values : ndarray
    axis : int, optional
    skipna : bool, default True
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    result : float64
        Unless input is a float array, in which case use the same
        precision as the input array.

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, np.nan, 1, 3, 2])
    >>> nanops.nankurt(s)
    -1.2892561983471076
    """
    ...

@disallow("M8", "m8")
@maybe_operate_rowwise
def nanprod(values: np.ndarray, *, axis: int | None = ..., skipna: bool = ..., min_count: int = ..., mask: npt.NDArray[np.bool_] | None = ...) -> float:
    """
    Parameters
    ----------
    values : ndarray[dtype]
    axis : int, optional
    skipna : bool, default True
    min_count: int, default 0
    mask : ndarray[bool], optional
        nan-mask if known

    Returns
    -------
    Dtype
        The product of all elements on a given axis. ( NaNs are treated as 1)

    Examples
    --------
    >>> import pandas.core.nanops as nanops
    >>> s = pd.Series([1, 2, 3, np.nan])
    >>> nanops.nanprod(s)
    6.0
    """
    ...

def check_below_min_count(shape: tuple[int, ...], mask: npt.NDArray[np.bool_] | None, min_count: int) -> bool:
    """
    Check for the `min_count` keyword. Returns True if below `min_count` (when
    missing value should be returned from the reduction).

    Parameters
    ----------
    shape : tuple
        The shape of the values (`values.shape`).
    mask : ndarray[bool] or None
        Boolean numpy array (typically of same shape as `shape`) or None.
    min_count : int
        Keyword passed through from sum/prod call.

    Returns
    -------
    bool
    """
    ...

@disallow("M8", "m8")
def nancorr(a: np.ndarray, b: np.ndarray, *, method=..., min_periods: int | None = ...) -> float:
    """
    a, b: ndarrays
    """
    ...

def get_corr_func(method) -> Callable[[np.ndarray, np.ndarray], float]:
    ...

@disallow("M8", "m8")
def nancov(a: np.ndarray, b: np.ndarray, *, min_periods: int | None = ..., ddof: int | None = ...) -> float:
    ...

def make_nancomp(op): # -> (x: Unknown, y: Unknown) -> Unknown:
    ...

nangt = ...
nange = ...
nanlt = ...
nanle = ...
naneq = ...
nanne = ...
def na_accum_func(values: ArrayLike, accum_func, *, skipna: bool) -> ArrayLike:
    """
    Cumulative function with skipna support.

    Parameters
    ----------
    values : np.ndarray or ExtensionArray
    accum_func : {np.cumprod, np.maximum.accumulate, np.cumsum, np.minimum.accumulate}
    skipna : bool

    Returns
    -------
    np.ndarray or ExtensionArray
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Any, TYPE_CHECKING
from pandas.core.computation import expr, ops, scope as _scope
from pandas.core.computation.expr import BaseExprVisitor
from pandas.core.indexes.base import Index

""" manage PyTables query interface via Expressions """
if TYPE_CHECKING:
    ...
class PyTablesScope(_scope.Scope):
    __slots__ = ...
    queryables: dict[str, Any]
    def __init__(self, level: int, global_dict=..., local_dict=..., queryables: dict[str, Any] | None = ...) -> None:
        ...
    


class Term(ops.Term):
    env: PyTablesScope
    def __new__(cls, name, env, side=..., encoding=...): # -> Self@Term | Constant:
        ...
    
    def __init__(self, name, env: PyTablesScope, side=..., encoding=...) -> None:
        ...
    
    @property
    def value(self):
        ...
    


class Constant(Term):
    def __init__(self, value, env: PyTablesScope, side=..., encoding=...) -> None:
        ...
    


class BinOp(ops.BinOp):
    _max_selectors = ...
    op: str
    queryables: dict[str, Any]
    condition: str | None
    def __init__(self, op: str, lhs, rhs, queryables: dict[str, Any], encoding) -> None:
        ...
    
    def prune(self, klass): # -> ConditionBinOp | FilterBinOp | JointConditionBinOp | JointFilterBinOp:
        ...
    
    def conform(self, rhs): # -> ndarray[Any, Unknown] | list[Unknown]:
        """inplace conform rhs"""
        ...
    
    @property
    def is_valid(self) -> bool:
        """return True if this is a valid field"""
        ...
    
    @property
    def is_in_table(self) -> bool:
        """
        return True if this is a valid column name for generation (e.g. an
        actual column in the table)
        """
        ...
    
    @property
    def kind(self): # -> Any | None:
        """the kind of my field"""
        ...
    
    @property
    def meta(self): # -> Any | None:
        """the meta of my field"""
        ...
    
    @property
    def metadata(self): # -> Any | None:
        """the metadata of my field"""
        ...
    
    def generate(self, v) -> str:
        """create and return the op string for this TermValue"""
        ...
    
    def convert_value(self, v) -> TermValue:
        """
        convert the expression that is in the term to something that is
        accepted by pytables
        """
        ...
    
    def convert_values(self): # -> None:
        ...
    


class FilterBinOp(BinOp):
    filter: tuple[Any, Any, Index] | None = ...
    def __repr__(self) -> str:
        ...
    
    def invert(self): # -> Self@FilterBinOp:
        """invert the filter"""
        ...
    
    def format(self): # -> list[tuple[Any, Any, Index] | None]:
        """return the actual filter format"""
        ...
    
    def evaluate(self): # -> Self@FilterBinOp | None:
        ...
    
    def generate_filter_op(self, invert: bool = ...): # -> (axis: Unknown, vals: Unknown) -> Unknown:
        ...
    


class JointFilterBinOp(FilterBinOp):
    def format(self):
        ...
    
    def evaluate(self): # -> Self@JointFilterBinOp:
        ...
    


class ConditionBinOp(BinOp):
    def __repr__(self) -> str:
        ...
    
    def invert(self):
        """invert the condition"""
        ...
    
    def format(self): # -> str | None:
        """return the actual ne format"""
        ...
    
    def evaluate(self): # -> Self@ConditionBinOp | None:
        ...
    


class JointConditionBinOp(ConditionBinOp):
    def evaluate(self): # -> Self@JointConditionBinOp:
        ...
    


class UnaryOp(ops.UnaryOp):
    def prune(self, klass): # -> None:
        ...
    


class PyTablesExprVisitor(BaseExprVisitor):
    const_type = Constant
    term_type = Term
    def __init__(self, env, engine, parser, **kwargs) -> None:
        ...
    
    def visit_UnaryOp(self, node, **kwargs): # -> UnaryOp | Term | None:
        ...
    
    def visit_Index(self, node, **kwargs): # -> Any:
        ...
    
    def visit_Assign(self, node, **kwargs): # -> Any:
        ...
    
    def visit_Subscript(self, node, **kwargs): # -> Term:
        ...
    
    def visit_Attribute(self, node, **kwargs): # -> term_type | Any:
        ...
    
    def translate_In(self, op): # -> Eq:
        ...
    


class PyTablesExpr(expr.Expr):
    """
    Hold a pytables-like expression, comprised of possibly multiple 'terms'.

    Parameters
    ----------
    where : string term expression, PyTablesExpr, or list-like of PyTablesExprs
    queryables : a "kinds" map (dict of column name -> kind), or None if column
        is non-indexable
    encoding : an encoding that will encode the query terms

    Returns
    -------
    a PyTablesExpr object

    Examples
    --------
    'index>=date'
    "columns=['A', 'D']"
    'columns=A'
    'columns==A'
    "~(columns=['A','B'])"
    'index>df.index[3] & string="bar"'
    '(index>df.index[3] & index<=df.index[6]) | string="bar"'
    "ts>=Timestamp('2012-02-01')"
    "major_axis>=20130101"
    """
    _visitor: PyTablesExprVisitor | None
    env: PyTablesScope
    expr: str
    def __init__(self, where, queryables: dict[str, Any] | None = ..., encoding=..., scope_level: int = ...) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    def evaluate(self): # -> tuple[Unknown, Unknown]:
        """create and return the numexpr condition and filter"""
        ...
    


class TermValue:
    """hold a term value the we use to construct a condition/filter"""
    def __init__(self, value, converted, kind: str) -> None:
        ...
    
    def tostring(self, encoding) -> str:
        """quote the string if not encoded else encode and return"""
        ...
    


def maybe_expression(s) -> bool:
    """loose checking if s is a pytables-acceptable expression"""
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from datetime import datetime, tzinfo
from typing import Callable

dateutil_gettz: Callable[[str], tzinfo]
def tz_standardize(tz: tzinfo) -> tzinfo:
    ...

def tz_compare(start: tzinfo | None, end: tzinfo | None) -> bool:
    ...

def infer_tzinfo(start: datetime | None, end: datetime | None) -> tzinfo | None:
    ...

def maybe_get_tz(tz: str | int | np.int64 | tzinfo | None) -> tzinfo | None:
    ...

def get_timezone(tz: tzinfo) -> tzinfo | str:
    ...

def is_utc(tz: tzinfo | None) -> bool:
    ...

def is_fixed_offset(tz: tzinfo) -> bool:
    ...




"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas._typing import ArrayLike, DtypeObj, Manager, Shape
from pandas.util._decorators import cache_readonly
from pandas.core.internals.blocks import Block
from pandas import Index

if TYPE_CHECKING:
    ...
def concat_arrays(to_concat: list) -> ArrayLike:
    """
    Alternative for concat_compat but specialized for use in the ArrayManager.

    Differences: only deals with 1D arrays (no axis keyword), assumes
    ensure_wrapped_if_datetimelike and does not skip empty arrays to determine
    the dtype.
    In addition ensures that all NullArrayProxies get replaced with actual
    arrays.

    Parameters
    ----------
    to_concat : list of arrays

    Returns
    -------
    np.ndarray or ExtensionArray
    """
    ...

def concatenate_managers(mgrs_indexers, axes: list[Index], concat_axis: int, copy: bool) -> Manager:
    """
    Concatenate block managers into one.

    Parameters
    ----------
    mgrs_indexers : list of (BlockManager, {axis: indexer,...}) tuples
    axes : list of Index
    concat_axis : int
    copy : bool

    Returns
    -------
    BlockManager
    """
    ...

class JoinUnit:
    def __init__(self, block: Block, shape: Shape, indexers=...) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    @cache_readonly
    def needs_filling(self) -> bool:
        ...
    
    @cache_readonly
    def dtype(self) -> DtypeObj:
        ...
    
    @cache_readonly
    def is_na(self) -> bool:
        ...
    
    def get_reindexed_values(self, empty_dtype: DtypeObj, upcasted_na) -> ArrayLike:
        ...
    





"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Callable, Iterable, Sequence, TYPE_CHECKING, final
from pandas._libs import internals as libinternals
from pandas._libs.internals import BlockPlacement
from pandas._typing import ArrayLike, DtypeObj, F, IgnoreRaise, Shape, npt
from pandas.util._decorators import cache_readonly
from pandas.core.arrays import DatetimeArray, ExtensionArray, TimedeltaArray
from pandas.core.base import PandasObject
from pandas import Float64Index, Index
from pandas.core.arrays._mixins import NDArrayBackedExtensionArray

if TYPE_CHECKING:
    ...
_dtype_obj = ...
def maybe_split(meth: F) -> F:
    """
    If we have a multi-column block, split and operate block-wise.  Otherwise
    use the original method.
    """
    ...

class Block(PandasObject):
    """
    Canonical n-dimensional unit of homogeneous dtype contained in a pandas
    data structure

    Index-ignorant; let the container take care of that
    """
    values: np.ndarray | ExtensionArray
    ndim: int
    __init__: Callable
    __slots__ = ...
    is_numeric = ...
    is_object = ...
    is_extension = ...
    _can_consolidate = ...
    _validate_ndim = ...
    @final
    @cache_readonly
    def is_categorical(self) -> bool:
        ...
    
    @final
    @property
    def is_bool(self) -> bool:
        """
        We can be bool if a) we are bool dtype or b) object dtype with bool objects.
        """
        ...
    
    @final
    def external_values(self): # -> ArrayLike:
        ...
    
    @final
    @cache_readonly
    def fill_value(self): # -> object | float | Literal[0, False]:
        ...
    
    @property
    def mgr_locs(self) -> BlockPlacement:
        ...
    
    @mgr_locs.setter
    def mgr_locs(self, new_mgr_locs: BlockPlacement) -> None:
        ...
    
    @final
    def make_block(self, values, placement=...) -> Block:
        """
        Create a new block, with type inference propagate any values that are
        not specified
        """
        ...
    
    @final
    def make_block_same_class(self, values, placement: BlockPlacement | None = ...) -> Block:
        """Wrap given values in a block of same type as self."""
        ...
    
    @final
    def __repr__(self) -> str:
        ...
    
    @final
    def __len__(self) -> int:
        ...
    
    @final
    def getitem_block(self, slicer: slice | npt.NDArray[np.intp]) -> Block:
        """
        Perform __getitem__-like, return result as block.

        Only supports slices that preserve dimensionality.
        """
        ...
    
    @final
    def getitem_block_columns(self, slicer: slice, new_mgr_locs: BlockPlacement) -> Block:
        """
        Perform __getitem__-like, return result as block.

        Only supports slices that preserve dimensionality.
        """
        ...
    
    @final
    def should_store(self, value: ArrayLike) -> bool:
        """
        Should we set self.values[indexer] = value inplace or do we need to cast?

        Parameters
        ----------
        value : np.ndarray or ExtensionArray

        Returns
        -------
        bool
        """
        ...
    
    @final
    def apply(self, func, **kwargs) -> list[Block]:
        """
        apply the function to my values; return a block if we are not
        one
        """
        ...
    
    def reduce(self, func, ignore_failures: bool = ...) -> list[Block]:
        ...
    
    @final
    def split_and_operate(self, func, *args, **kwargs) -> list[Block]:
        """
        Split the block and apply func column-by-column.

        Parameters
        ----------
        func : Block method
        *args
        **kwargs

        Returns
        -------
        List[Block]
        """
        ...
    
    @final
    def coerce_to_target_dtype(self, other) -> Block:
        """
        coerce the current block to a dtype compat for other
        we will return a block, possibly object, and not raise

        we can also safely try to coerce to the same dtype
        and will receive the same block
        """
        ...
    
    def convert(self, copy: bool = ..., datetime: bool = ..., numeric: bool = ..., timedelta: bool = ...) -> list[Block]:
        """
        attempt to coerce any object types to better types return a copy
        of the block (if copy = True) by definition we are not an ObjectBlock
        here!
        """
        ...
    
    @cache_readonly
    def dtype(self) -> DtypeObj:
        ...
    
    @final
    def astype(self, dtype: DtypeObj, copy: bool = ..., errors: IgnoreRaise = ...) -> Block:
        """
        Coerce to the new dtype.

        Parameters
        ----------
        dtype : np.dtype or ExtensionDtype
        copy : bool, default False
            copy if indicated
        errors : str, {'raise', 'ignore'}, default 'raise'
            - ``raise`` : allow exceptions to be raised
            - ``ignore`` : suppress exceptions. On error return original object

        Returns
        -------
        Block
        """
        ...
    
    @final
    def to_native_types(self, na_rep=..., quoting=..., **kwargs) -> Block:
        """convert to our native types format"""
        ...
    
    @final
    def copy(self, deep: bool = ...) -> Block:
        """copy constructor"""
        ...
    
    @final
    def replace(self, to_replace, value, inplace: bool = ..., mask: npt.NDArray[np.bool_] | None = ...) -> list[Block]:
        """
        replace the to_replace value with value, possible to create new
        blocks here this is just a call to putmask.
        """
        ...
    
    @final
    def replace_list(self, src_list: Iterable[Any], dest_list: Sequence[Any], inplace: bool = ..., regex: bool = ...) -> list[Block]:
        """
        See BlockManager.replace_list docstring.
        """
        ...
    
    @property
    def shape(self) -> Shape:
        ...
    
    def iget(self, i: int | tuple[int, int] | tuple[slice, int]) -> np.ndarray:
        ...
    
    def set_inplace(self, locs, values: ArrayLike, copy: bool = ...) -> None:
        """
        Modify block values in-place with new item value.

        If copy=True, first copy the underlying values in place before modifying
        (for Copy-on-Write).

        Notes
        -----
        `set_inplace` never creates a new array or new Block, whereas `setitem`
        _may_ create a new array and always creates a new Block.

        Caller is responsible for checking values.dtype == self.dtype.
        """
        ...
    
    def take_nd(self, indexer: npt.NDArray[np.intp], axis: int, new_mgr_locs: BlockPlacement | None = ..., fill_value=...) -> Block:
        """
        Take values according to indexer and return them as a block.
        """
        ...
    
    def setitem(self, indexer, value) -> Block:
        """
        Attempt self.values[indexer] = value, possibly creating a new array.

        Parameters
        ----------
        indexer : tuple, list-like, array-like, slice, int
            The subset of self.values to set
        value : object
            The value being set

        Returns
        -------
        Block

        Notes
        -----
        `indexer` is a direct slice/positional indexer. `value` must
        be a compatible shape.
        """
        ...
    
    def putmask(self, mask, new) -> list[Block]:
        """
        putmask the data to the block; it is possible that we may create a
        new dtype of block

        Return the resulting block(s).

        Parameters
        ----------
        mask : np.ndarray[bool], SparseArray[bool], or BooleanArray
        new : a ndarray/object

        Returns
        -------
        List[Block]
        """
        ...
    
    def where(self, other, cond, _downcast=...) -> list[Block]:
        """
        evaluate the block; return result block(s) from the result

        Parameters
        ----------
        other : a ndarray/object
        cond : np.ndarray[bool], SparseArray[bool], or BooleanArray
        _downcast : str or None, default "infer"
            Private because we only specify it when calling from fillna.

        Returns
        -------
        List[Block]
        """
        ...
    
    def fillna(self, value, limit: int | None = ..., inplace: bool = ..., downcast=...) -> list[Block]:
        """
        fillna on the block with the value. If we fail, then convert to
        ObjectBlock and try again
        """
        ...
    
    def interpolate(self, method: str = ..., axis: int = ..., index: Index | None = ..., inplace: bool = ..., limit: int | None = ..., limit_direction: str = ..., limit_area: str | None = ..., fill_value: Any | None = ..., downcast: str | None = ..., **kwargs) -> list[Block]:
        ...
    
    def diff(self, n: int, axis: int = ...) -> list[Block]:
        """return block for the diff of the values"""
        ...
    
    def shift(self, periods: int, axis: int = ..., fill_value: Any = ...) -> list[Block]:
        """shift the block by periods, possibly upcast"""
        ...
    
    @final
    def quantile(self, qs: Float64Index, interpolation=..., axis: int = ...) -> Block:
        """
        compute the quantiles of the

        Parameters
        ----------
        qs : Float64Index
            List of the quantiles to be computed.
        interpolation : str, default 'linear'
            Type of interpolation.
        axis : int, default 0
            Axis to compute.

        Returns
        -------
        Block
        """
        ...
    
    def delete(self, loc) -> Block:
        """
        Return a new Block with the given loc(s) deleted.
        """
        ...
    
    @property
    def is_view(self) -> bool:
        """return a boolean if I am possibly a view"""
        ...
    
    @property
    def array_values(self) -> ExtensionArray:
        """
        The array that Series.array returns. Always an ExtensionArray.
        """
        ...
    
    def get_values(self, dtype: DtypeObj | None = ...) -> np.ndarray:
        """
        return an internal format, currently just the ndarray
        this is often overridden to handle to_dense like operations
        """
        ...
    
    def values_for_json(self) -> np.ndarray:
        ...
    


class EABackedBlock(Block):
    """
    Mixin for Block subclasses backed by ExtensionArray.
    """
    values: ExtensionArray
    def setitem(self, indexer, value): # -> Self@EABackedBlock | Block:
        """
        Attempt self.values[indexer] = value, possibly creating a new array.

        This differs from Block.setitem by not allowing setitem to change
        the dtype of the Block.

        Parameters
        ----------
        indexer : tuple, list-like, array-like, slice, int
            The subset of self.values to set
        value : object
            The value being set

        Returns
        -------
        Block

        Notes
        -----
        `indexer` is a direct slice/positional indexer. `value` must
        be a compatible shape.
        """
        ...
    
    def where(self, other, cond, _downcast=...) -> list[Block]:
        ...
    
    def putmask(self, mask, new) -> list[Block]:
        """
        See Block.putmask.__doc__
        """
        ...
    
    def fillna(self, value, limit: int | None = ..., inplace: bool = ..., downcast=...) -> list[Block]:
        ...
    
    def delete(self, loc) -> Block:
        ...
    
    @cache_readonly
    def array_values(self) -> ExtensionArray:
        ...
    
    def get_values(self, dtype: DtypeObj | None = ...) -> np.ndarray:
        """
        return object dtype as boxed values, such as Timestamps/Timedelta
        """
        ...
    
    def values_for_json(self) -> np.ndarray:
        ...
    
    def interpolate(self, method=..., axis=..., inplace=..., limit=..., fill_value=..., **kwargs): # -> Block:
        ...
    


class ExtensionBlock(libinternals.Block, EABackedBlock):
    """
    Block for holding extension types.

    Notes
    -----
    This holds all 3rd-party extension array types. It's also the immediate
    parent class for our internal extension types' blocks, CategoricalBlock.

    ExtensionArrays are limited to 1-D.
    """
    _can_consolidate = ...
    _validate_ndim = ...
    is_extension = ...
    values: ExtensionArray
    @cache_readonly
    def shape(self) -> Shape:
        ...
    
    def iget(self, i: int | tuple[int, int] | tuple[slice, int]): # -> ExtensionArray | Any:
        ...
    
    def set_inplace(self, locs, values: ArrayLike, copy: bool = ...) -> None:
        ...
    
    @property
    def is_view(self) -> bool:
        """Extension arrays are never treated as views."""
        ...
    
    @cache_readonly
    def is_numeric(self): # -> bool:
        ...
    
    def take_nd(self, indexer: npt.NDArray[np.intp], axis: int = ..., new_mgr_locs: BlockPlacement | None = ..., fill_value=...) -> Block:
        """
        Take values according to indexer and return them as a block.
        """
        ...
    
    @final
    def getitem_block_index(self, slicer: slice) -> ExtensionBlock:
        """
        Perform __getitem__-like specialized to slicing along index.
        """
        ...
    
    def diff(self, n: int, axis: int = ...) -> list[Block]:
        ...
    
    def shift(self, periods: int, axis: int = ..., fill_value: Any = ...) -> list[Block]:
        """
        Shift the block by `periods`.

        Dispatches to underlying ExtensionArray and re-boxes in an
        ExtensionBlock.
        """
        ...
    


class NumpyBlock(libinternals.NumpyBlock, Block):
    values: np.ndarray
    @property
    def is_view(self) -> bool:
        """return a boolean if I am possibly a view"""
        ...
    
    @property
    def array_values(self) -> ExtensionArray:
        ...
    
    def get_values(self, dtype: DtypeObj | None = ...) -> np.ndarray:
        ...
    
    def values_for_json(self) -> np.ndarray:
        ...
    
    def delete(self, loc) -> Block:
        ...
    


class NumericBlock(NumpyBlock):
    __slots__ = ...
    is_numeric = ...


class NDArrayBackedExtensionBlock(libinternals.NDArrayBackedBlock, EABackedBlock):
    """
    Block backed by an NDArrayBackedExtensionArray
    """
    values: NDArrayBackedExtensionArray
    @cache_readonly
    def is_extension(self) -> bool:
        ...
    
    @property
    def is_view(self) -> bool:
        """return a boolean if I am possibly a view"""
        ...
    
    def diff(self, n: int, axis: int = ...) -> list[Block]:
        """
        1st discrete difference.

        Parameters
        ----------
        n : int
            Number of periods to diff.
        axis : int, default 0
            Axis to diff upon.

        Returns
        -------
        A list with a new Block.

        Notes
        -----
        The arguments here are mimicking shift so they are called correctly
        by apply.
        """
        ...
    
    def shift(self, periods: int, axis: int = ..., fill_value: Any = ...) -> list[Block]:
        ...
    


class DatetimeLikeBlock(NDArrayBackedExtensionBlock):
    """Block for datetime64[ns], timedelta64[ns]."""
    __slots__ = ...
    is_numeric = ...
    values: DatetimeArray | TimedeltaArray
    def values_for_json(self) -> np.ndarray:
        ...
    


class DatetimeTZBlock(DatetimeLikeBlock):
    """implement a datetime64 block with a tz attribute"""
    values: DatetimeArray
    __slots__ = ...
    is_extension = ...
    _validate_ndim = ...
    _can_consolidate = ...
    values_for_json = ...


class ObjectBlock(NumpyBlock):
    __slots__ = ...
    is_object = ...
    @maybe_split
    def reduce(self, func, ignore_failures: bool = ...) -> list[Block]:
        """
        For object-dtype, we operate column-wise.
        """
        ...
    
    @maybe_split
    def convert(self, copy: bool = ..., datetime: bool = ..., numeric: bool = ..., timedelta: bool = ...) -> list[Block]:
        """
        attempt to cast any object types to better types return a copy of
        the block (if copy = True) by definition we ARE an ObjectBlock!!!!!
        """
        ...
    


class CategoricalBlock(ExtensionBlock):
    __slots__ = ...
    @property
    def dtype(self) -> DtypeObj:
        ...
    


def maybe_coerce_values(values: ArrayLike) -> ArrayLike:
    """
    Input validation for values passed to __init__. Ensure that
    any datetime64/timedelta64 dtypes are in nanoseconds.  Ensure
    that we do not have string dtypes.

    Parameters
    ----------
    values : np.ndarray or ExtensionArray

    Returns
    -------
    values : np.ndarray or ExtensionArray
    """
    ...

def get_block_type(dtype: DtypeObj): # -> Type[ExtensionBlock] | Type[CategoricalBlock] | Type[DatetimeTZBlock] | Type[NDArrayBackedExtensionBlock] | Type[DatetimeLikeBlock] | Type[NumericBlock] | Type[ObjectBlock]:
    """
    Find the appropriate Block subclass to use for the given values and dtype.

    Parameters
    ----------
    dtype : numpy or pandas dtype

    Returns
    -------
    cls : class, subclass of Block
    """
    ...

def new_block_2d(values: ArrayLike, placement: BlockPlacement): # -> ExtensionBlock | CategoricalBlock | DatetimeTZBlock | NDArrayBackedExtensionBlock | DatetimeLikeBlock | NumericBlock | ObjectBlock:
    ...

def new_block(values, placement, *, ndim: int) -> Block:
    ...

def check_ndim(values, placement: BlockPlacement, ndim: int) -> None:
    """
    ndim inference and validation.

    Validates that values.ndim and ndim are consistent.
    Validates that len(values) and len(placement) are consistent.

    Parameters
    ----------
    values : array-like
    placement : BlockPlacement
    ndim : int

    Raises
    ------
    ValueError : the number of dimensions do not match
    """
    ...

def extract_pandas_array(values: np.ndarray | ExtensionArray, dtype: DtypeObj | None, ndim: int) -> tuple[np.ndarray | ExtensionArray, DtypeObj | None]:
    """
    Ensure that we don't allow PandasArray / PandasDtype in internals.
    """
    ...

def extend_blocks(result, blocks=...) -> list[Block]:
    """return a new extended blocks, given the result"""
    ...

def ensure_block_shape(values: ArrayLike, ndim: int = ...) -> ArrayLike:
    """
    Reshape if possible to have values.ndim == ndim.
    """
    ...

def to_native_types(values: ArrayLike, *, na_rep=..., quoting=..., float_format=..., decimal=..., **kwargs) -> np.ndarray:
    """convert to our native types format"""
    ...

def external_values(values: ArrayLike) -> ArrayLike:
    """
    The array that Series.values returns (public attribute).

    This has some historical constraints, and is overridden in block
    subclasses to return the correct array (e.g. period returns
    object ndarray and datetimetz a datetime64[ns] ndarray instead of
    proper extension array).
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Union
from pandas._libs import missing as libmissing
from pandas._typing import NpDtype, npt
from pandas.compat import pa_version_under1p01
from pandas.core.arrays.arrow import ArrowExtensionArray
from pandas.core.arrays.string_ import BaseStringArray, StringDtype
from pandas.core.strings.object_array import ObjectStringArrayMixin

if notpa_version_under1p01:
    ...
ArrowStringScalarOrNAT = Union[str, libmissing.NAType]
class ArrowStringArray(ArrowExtensionArray, BaseStringArray, ObjectStringArrayMixin):
    """
    Extension array for string data in a ``pyarrow.ChunkedArray``.

    .. versionadded:: 1.2.0

    .. warning::

       ArrowStringArray is considered experimental. The implementation and
       parts of the API may change without warning.

    Parameters
    ----------
    values : pyarrow.Array or pyarrow.ChunkedArray
        The array of data.

    Attributes
    ----------
    None

    Methods
    -------
    None

    See Also
    --------
    array
        The recommended function for creating a ArrowStringArray.
    Series.str
        The string methods are available on Series backed by
        a ArrowStringArray.

    Notes
    -----
    ArrowStringArray returns a BooleanArray for comparison methods.

    Examples
    --------
    >>> pd.array(['This is', 'some text', None, 'data.'], dtype="string[pyarrow]")
    <ArrowStringArray>
    ['This is', 'some text', <NA>, 'data.']
    Length: 4, dtype: string
    """
    _dtype: StringDtype
    def __init__(self, values) -> None:
        ...
    
    @property
    def dtype(self) -> StringDtype:
        """
        An instance of 'string[pyarrow]'.
        """
        ...
    
    def __array__(self, dtype: NpDtype | None = ...) -> np.ndarray:
        """Correctly construct numpy arrays when passed to `np.asarray()`."""
        ...
    
    def to_numpy(self, dtype: npt.DTypeLike | None = ..., copy: bool = ..., na_value=...) -> np.ndarray:
        """
        Convert to a NumPy ndarray.
        """
        ...
    
    def insert(self, loc: int, item) -> ArrowStringArray:
        ...
    
    def isin(self, values) -> npt.NDArray[np.bool_]:
        ...
    
    def astype(self, dtype, copy: bool = ...): # -> ArrayLike | Self@ArrowStringArray | BaseMaskedArray:
        ...
    
    _str_na_value = ...





"""
This type stub file was generated by pyright.
"""

import functools
import numpy as np
from typing import Any, Callable
from pandas._typing import Scalar

"""Common utilities for Numba operations with groupby ops"""
def validate_udf(func: Callable) -> None:
    """
    Validate user defined function for ops when using Numba with groupby ops.

    The first signature arguments should include:

    def f(values, index, ...):
        ...

    Parameters
    ----------
    func : function, default False
        user defined function

    Returns
    -------
    None

    Raises
    ------
    NumbaUtilError
    """
    ...

@functools.lru_cache(maxsize=None)
def generate_numba_agg_func(func: Callable[..., Scalar], nopython: bool, nogil: bool, parallel: bool) -> Callable[[np.ndarray, np.ndarray, np.ndarray, np.ndarray, int, Any], np.ndarray]:
    """
    Generate a numba jitted agg function specified by values from engine_kwargs.

    1. jit the user's function
    2. Return a groupby agg function with the jitted function inline

    Configurations specified in engine_kwargs apply to both the user's
    function _AND_ the groupby evaluation loop.

    Parameters
    ----------
    func : function
        function to be applied to each group and will be JITed
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit

    Returns
    -------
    Numba function
    """
    ...

@functools.lru_cache(maxsize=None)
def generate_numba_transform_func(func: Callable[..., np.ndarray], nopython: bool, nogil: bool, parallel: bool) -> Callable[[np.ndarray, np.ndarray, np.ndarray, np.ndarray, int, Any], np.ndarray]:
    """
    Generate a numba jitted transform function specified by values from engine_kwargs.

    1. jit the user's function
    2. Return a groupby transform function with the jitted function inline

    Configurations specified in engine_kwargs apply to both the user's
    function _AND_ the groupby evaluation loop.

    Parameters
    ----------
    func : function
        function to be applied to each window and will be JITed
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit

    Returns
    -------
    Numba function
    """
    ...




"""
This type stub file was generated by pyright.
"""

"""
Unopinionated display configuration.
"""
_initial_defencoding: str | None = ...
def detect_console_encoding() -> str:
    """
    Try to find the most capable encoding supported by the console.
    slightly modified from the way IPython handles the same issue.
    """
    ...

pc_encoding_doc = ...



"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any
from pandas._typing import ArrayLike, npt

"""
EA-compatible analogue to np.putmask
"""
def putmask_inplace(values: ArrayLike, mask: npt.NDArray[np.bool_], value: Any) -> None:
    """
    ExtensionArray-compatible implementation of np.putmask.  The main
    difference is we do not handle repeating or truncating like numpy.

    Parameters
    ----------
    values: np.ndarray or ExtensionArray
    mask : np.ndarray[bool]
        We assume extract_bool_array has already been called.
    value : Any
    """
    ...

def putmask_without_repeat(values: np.ndarray, mask: npt.NDArray[np.bool_], new: Any) -> None:
    """
    np.putmask will truncate or repeat if `new` is a listlike with
    len(new) != len(values).  We require an exact match.

    Parameters
    ----------
    values : np.ndarray
    mask : np.ndarray[bool]
    new : Any
    """
    ...

def validate_putmask(values: ArrayLike, mask: np.ndarray) -> tuple[npt.NDArray[np.bool_], bool]:
    """
    Validate mask and check if this putmask operation is a no-op.
    """
    ...

def extract_bool_array(mask: ArrayLike) -> npt.NDArray[np.bool_]:
    """
    If we have a SparseArray or BooleanArray, convert it to ndarray[bool].
    """
    ...

def setitem_datetimelike_compat(values: np.ndarray, num_set: int, other): # -> list[Unknown]:
    """
    Parameters
    ----------
    values : np.ndarray
    num_set : int
        For putmask, this is mask.sum()
    other : Any
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.arrays.arrow import ArrowExtensionArray
from pandas.core.arrays.base import ExtensionArray, ExtensionOpsMixin, ExtensionScalarOpsMixin
from pandas.core.arrays.boolean import BooleanArray
from pandas.core.arrays.categorical import Categorical
from pandas.core.arrays.datetimes import DatetimeArray
from pandas.core.arrays.floating import FloatingArray
from pandas.core.arrays.integer import IntegerArray
from pandas.core.arrays.interval import IntervalArray
from pandas.core.arrays.masked import BaseMaskedArray
from pandas.core.arrays.numpy_ import PandasArray
from pandas.core.arrays.period import PeriodArray, period_array
from pandas.core.arrays.sparse import SparseArray
from pandas.core.arrays.string_ import StringArray
from pandas.core.arrays.string_arrow import ArrowStringArray
from pandas.core.arrays.timedeltas import TimedeltaArray

__all__ = ["ArrowExtensionArray", "ExtensionArray", "ExtensionOpsMixin", "ExtensionScalarOpsMixin", "ArrowStringArray", "BaseMaskedArray", "BooleanArray", "Categorical", "DatetimeArray", "FloatingArray", "IntegerArray", "IntervalArray", "PandasArray", "PeriodArray", "period_array", "SparseArray", "StringArray", "TimedeltaArray"]



"""
This type stub file was generated by pyright.
"""

import numpy as np

"""
Templates for invalid operations.
"""
def invalid_comparison(left, right, op) -> np.ndarray:
    """
    If a comparison has mismatched types and is not necessarily meaningful,
    follow python3 conventions by:

        - returning all-False for equality
        - returning all-True for inequality
        - raising TypeError otherwise

    Parameters
    ----------
    left : array-like
    right : scalar, array-like
    op : operator.{eq, ne, lt, le, gt}

    Raises
    ------
    TypeError : on inequality comparisons
    """
    ...

def make_invalid_op(name: str): # -> (self: Unknown, other: Unknown | None = None) -> NoReturn:
    """
    Return a binary method that always raises a TypeError.

    Parameters
    ----------
    name : str

    Returns
    -------
    invalid_op : function
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from datetime import datetime
from pandas._libs.tslibs.offsets import BaseOffset
from pandas._typing import npt

class DateParseError(ValueError):
    ...


def parse_datetime_string(date_string: str, dayfirst: bool = ..., yearfirst: bool = ..., **kwargs) -> datetime:
    ...

def parse_time_string(arg: str, freq: BaseOffset | str | None = ..., dayfirst: bool | None = ..., yearfirst: bool | None = ...) -> tuple[datetime, str]:
    ...

def quarter_to_myear(year: int, quarter: int, freq: str) -> tuple[int, int]:
    ...

def try_parse_dates(values: npt.NDArray[np.object_], parser=..., dayfirst: bool = ..., default: datetime | None = ...) -> npt.NDArray[np.object_]:
    ...

def try_parse_date_and_time(dates: npt.NDArray[np.object_], times: npt.NDArray[np.object_], date_parser=..., time_parser=..., dayfirst: bool = ..., default: datetime | None = ...) -> npt.NDArray[np.object_]:
    ...

def try_parse_year_month_day(years: npt.NDArray[np.object_], months: npt.NDArray[np.object_], days: npt.NDArray[np.object_]) -> npt.NDArray[np.object_]:
    ...

def try_parse_datetime_components(years: npt.NDArray[np.object_], months: npt.NDArray[np.object_], days: npt.NDArray[np.object_], hours: npt.NDArray[np.object_], minutes: npt.NDArray[np.object_], seconds: npt.NDArray[np.object_]) -> npt.NDArray[np.object_]:
    ...

def format_is_iso(f: str) -> bool:
    ...

def guess_datetime_format(dt_str, dayfirst: bool | None = ...) -> str | None:
    ...

def concat_date_cols(date_cols: tuple, keep_trivial_numbers: bool = ...) -> npt.NDArray[np.object_]:
    ...

def get_rule_month(source: str) -> str:
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.arrays.sparse.accessor import SparseAccessor, SparseFrameAccessor
from pandas.core.arrays.sparse.array import BlockIndex, IntIndex, SparseArray, make_sparse_index
from pandas.core.arrays.sparse.dtype import SparseDtype

__all__ = ["BlockIndex", "IntIndex", "make_sparse_index", "SparseAccessor", "SparseArray", "SparseDtype", "SparseFrameAccessor"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
import pyarrow as pa
from typing import Any, TYPE_CHECKING, TypeVar
from pandas._libs import lib
from pandas._typing import Dtype, PositionalIndexer, TakeIndexer, npt
from pandas.compat import pa_version_under1p01
from pandas.util._decorators import deprecate_nonkeyword_arguments, doc
from pandas.core.arraylike import OpsMixin
from pandas.core.arrays.base import ExtensionArray
from pandas.core.arrays.arrow.dtype import ArrowDtype
from pandas import Series

if notpa_version_under1p01:
    ARROW_CMP_FUNCS = ...
    ARROW_LOGICAL_FUNCS = ...
    def cast_for_truediv(arrow_array: pa.ChunkedArray, pa_object: pa.Array | pa.Scalar) -> pa.ChunkedArray:
        ...
    
    def floordiv_compat(left: pa.ChunkedArray | pa.Array | pa.Scalar, right: pa.ChunkedArray | pa.Array | pa.Scalar) -> pa.ChunkedArray:
        ...
    
    ARROW_ARITHMETIC_FUNCS = ...
if TYPE_CHECKING:
    ...
ArrowExtensionArrayT = TypeVar("ArrowExtensionArrayT", bound="ArrowExtensionArray")
def to_pyarrow_type(dtype: ArrowDtype | pa.DataType | Dtype | None) -> pa.DataType | None:
    """
    Convert dtype to a pyarrow type instance.
    """
    ...

class ArrowExtensionArray(OpsMixin, ExtensionArray):
    """
    Pandas ExtensionArray backed by a PyArrow ChunkedArray.

    .. warning::

       ArrowExtensionArray is considered experimental. The implementation and
       parts of the API may change without warning.

    Parameters
    ----------
    values : pyarrow.Array or pyarrow.ChunkedArray

    Attributes
    ----------
    None

    Methods
    -------
    None

    Returns
    -------
    ArrowExtensionArray

    Notes
    -----
    Most methods are implemented using `pyarrow compute functions. <https://arrow.apache.org/docs/python/api/compute.html>`__
    Some methods may either raise an exception or raise a ``PerformanceWarning`` if an
    associated compute function is not available based on the installed version of PyArrow.

    Please install the latest version of PyArrow to enable the best functionality and avoid
    potential bugs in prior versions of PyArrow.

    Examples
    --------
    Create an ArrowExtensionArray with :func:`pandas.array`:

    >>> pd.array([1, 1, None], dtype="int64[pyarrow]")
    <ArrowExtensionArray>
    [1, 1, <NA>]
    Length: 3, dtype: int64[pyarrow]
    """
    _data: pa.ChunkedArray
    _dtype: ArrowDtype
    def __init__(self, values: pa.Array | pa.ChunkedArray) -> None:
        ...
    
    def __getitem__(self, item: PositionalIndexer): # -> Self@ArrowExtensionArray | ArrowExtensionArray | NAType:
        """Select a subset of self.

        Parameters
        ----------
        item : int, slice, or ndarray
            * int: The position in 'self' to get.
            * slice: A slice object, where 'start', 'stop', and 'step' are
              integers or None
            * ndarray: A 1-d boolean NumPy ndarray the same length as 'self'

        Returns
        -------
        item : scalar or ExtensionArray

        Notes
        -----
        For scalar ``item``, return a scalar value suitable for the array's
        type. This should be an instance of ``self.dtype.type``.
        For slice ``key``, return an instance of ``ExtensionArray``, even
        if the slice is length 0 or 1.
        For a boolean mask, return an instance of ``ExtensionArray``, filtered
        to the values where ``item`` is True.
        """
        ...
    
    def __arrow_array__(self, type=...):
        """Convert myself to a pyarrow ChunkedArray."""
        ...
    
    def __invert__(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        ...
    
    def __neg__(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        ...
    
    def __pos__(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        ...
    
    def __abs__(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        ...
    
    def equals(self, other) -> bool:
        ...
    
    @property
    def dtype(self) -> ArrowDtype:
        """
        An instance of 'ExtensionDtype'.
        """
        ...
    
    @property
    def nbytes(self) -> int:
        """
        The number of bytes needed to store this object in memory.
        """
        ...
    
    def __len__(self) -> int:
        """
        Length of this array.

        Returns
        -------
        length : int
        """
        ...
    
    def isna(self) -> npt.NDArray[np.bool_]:
        """
        Boolean NumPy array indicating if each value is missing.

        This should return a 1-D array the same length as 'self'.
        """
        ...
    
    @deprecate_nonkeyword_arguments(version=None, allowed_args=["self"])
    def argsort(self, ascending: bool = ..., kind: str = ..., na_position: str = ..., *args, **kwargs) -> np.ndarray:
        ...
    
    def argmin(self, skipna: bool = ...) -> int:
        ...
    
    def argmax(self, skipna: bool = ...) -> int:
        ...
    
    def copy(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        """
        Return a shallow copy of the array.

        Underlying ChunkedArray is immutable, so a deep copy is unnecessary.

        Returns
        -------
        type(self)
        """
        ...
    
    def dropna(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        """
        Return ArrowExtensionArray without NA values.

        Returns
        -------
        ArrowExtensionArray
        """
        ...
    
    def isin(self, values) -> npt.NDArray[np.bool_]:
        ...
    
    @doc(ExtensionArray.factorize)
    def factorize(self, na_sentinel: int | lib.NoDefault = ..., use_na_sentinel: bool | lib.NoDefault = ...) -> tuple[np.ndarray, ExtensionArray]:
        ...
    
    def reshape(self, *args, **kwargs):
        ...
    
    def take(self, indices: TakeIndexer, allow_fill: bool = ..., fill_value: Any = ...) -> ArrowExtensionArray:
        """
        Take elements from an array.

        Parameters
        ----------
        indices : sequence of int or one-dimensional np.ndarray of int
            Indices to be taken.
        allow_fill : bool, default False
            How to handle negative values in `indices`.

            * False: negative values in `indices` indicate positional indices
              from the right (the default). This is similar to
              :func:`numpy.take`.

            * True: negative values in `indices` indicate
              missing values. These values are set to `fill_value`. Any other
              other negative values raise a ``ValueError``.

        fill_value : any, optional
            Fill value to use for NA-indices when `allow_fill` is True.
            This may be ``None``, in which case the default NA value for
            the type, ``self.dtype.na_value``, is used.

            For many ExtensionArrays, there will be two representations of
            `fill_value`: a user-facing "boxed" scalar, and a low-level
            physical NA value. `fill_value` should be the user-facing version,
            and the implementation should handle translating that to the
            physical version for processing the take if necessary.

        Returns
        -------
        ExtensionArray

        Raises
        ------
        IndexError
            When the indices are out of bounds for the array.
        ValueError
            When `indices` contains negative values other than ``-1``
            and `allow_fill` is True.

        See Also
        --------
        numpy.take
        api.extensions.take

        Notes
        -----
        ExtensionArray.take is called by ``Series.__getitem__``, ``.loc``,
        ``iloc``, when `indices` is a sequence of values. Additionally,
        it's called by :meth:`Series.reindex`, or any other method
        that causes realignment, with a `fill_value`.
        """
        ...
    
    def unique(self: ArrowExtensionArrayT) -> ArrowExtensionArrayT:
        """
        Compute the ArrowExtensionArray of unique values.

        Returns
        -------
        ArrowExtensionArray
        """
        ...
    
    def value_counts(self, dropna: bool = ...) -> Series:
        """
        Return a Series containing counts of each unique value.

        Parameters
        ----------
        dropna : bool, default True
            Don't include counts of missing values.

        Returns
        -------
        counts : Series

        See Also
        --------
        Series.value_counts
        """
        ...
    
    def __setitem__(self, key: int | slice | np.ndarray, value: Any) -> None:
        """Set one or more values inplace.

        Parameters
        ----------
        key : int, ndarray, or slice
            When called from, e.g. ``Series.__setitem__``, ``key`` will be
            one of

            * scalar int
            * ndarray of integers.
            * boolean ndarray
            * slice object

        value : ExtensionDtype.type, Sequence[ExtensionDtype.type], or object
            value or values to be set of ``key``.

        Returns
        -------
        None
        """
        ...
    





"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

def hash_object_array(arr: npt.NDArray[np.object_], key: str, encoding: str = ...) -> npt.NDArray[np.uint64]:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Iterable, Sequence, TypeVar, overload

"""
Module that contains many useful utilities
for validating data or function arguments
"""
BoolishT = TypeVar("BoolishT", bool, int)
BoolishNoneT = TypeVar("BoolishNoneT", bool, int, None)
def validate_args(fname, args, max_fname_arg_count, compat_args) -> None:
    """
    Checks whether the length of the `*args` argument passed into a function
    has at most `len(compat_args)` arguments and whether or not all of these
    elements in `args` are set to their default values.

    Parameters
    ----------
    fname : str
        The name of the function being passed the `*args` parameter
    args : tuple
        The `*args` parameter passed into a function
    max_fname_arg_count : int
        The maximum number of arguments that the function `fname`
        can accept, excluding those in `args`. Used for displaying
        appropriate error messages. Must be non-negative.
    compat_args : dict
        A dictionary of keys and their associated default values.
        In order to accommodate buggy behaviour in some versions of `numpy`,
        where a signature displayed keyword arguments but then passed those
        arguments **positionally** internally when calling downstream
        implementations, a dict ensures that the original
        order of the keyword arguments is enforced.

    Raises
    ------
    TypeError
        If `args` contains more values than there are `compat_args`
    ValueError
        If `args` contains values that do not correspond to those
        of the default values specified in `compat_args`
    """
    ...

def validate_kwargs(fname, kwargs, compat_args) -> None:
    """
    Checks whether parameters passed to the **kwargs argument in a
    function `fname` are valid parameters as specified in `*compat_args`
    and whether or not they are set to their default values.

    Parameters
    ----------
    fname : str
        The name of the function being passed the `**kwargs` parameter
    kwargs : dict
        The `**kwargs` parameter passed into `fname`
    compat_args: dict
        A dictionary of keys that `kwargs` is allowed to have and their
        associated default values

    Raises
    ------
    TypeError if `kwargs` contains keys not in `compat_args`
    ValueError if `kwargs` contains keys in `compat_args` that do not
    map to the default values specified in `compat_args`
    """
    ...

def validate_args_and_kwargs(fname, args, kwargs, max_fname_arg_count, compat_args) -> None:
    """
    Checks whether parameters passed to the *args and **kwargs argument in a
    function `fname` are valid parameters as specified in `*compat_args`
    and whether or not they are set to their default values.

    Parameters
    ----------
    fname: str
        The name of the function being passed the `**kwargs` parameter
    args: tuple
        The `*args` parameter passed into a function
    kwargs: dict
        The `**kwargs` parameter passed into `fname`
    max_fname_arg_count: int
        The minimum number of arguments that the function `fname`
        requires, excluding those in `args`. Used for displaying
        appropriate error messages. Must be non-negative.
    compat_args: dict
        A dictionary of keys that `kwargs` is allowed to
        have and their associated default values.

    Raises
    ------
    TypeError if `args` contains more values than there are
    `compat_args` OR `kwargs` contains keys not in `compat_args`
    ValueError if `args` contains values not at the default value (`None`)
    `kwargs` contains keys in `compat_args` that do not map to the default
    value as specified in `compat_args`

    See Also
    --------
    validate_args : Purely args validation.
    validate_kwargs : Purely kwargs validation.

    """
    ...

def validate_bool_kwarg(value: BoolishNoneT, arg_name, none_allowed=..., int_allowed=...) -> BoolishNoneT:
    """
    Ensure that argument passed in arg_name can be interpreted as boolean.

    Parameters
    ----------
    value : bool
        Value to be validated.
    arg_name : str
        Name of the argument. To be reflected in the error message.
    none_allowed : bool, default True
        Whether to consider None to be a valid boolean.
    int_allowed : bool, default False
        Whether to consider integer value to be a valid boolean.

    Returns
    -------
    value
        The same value as input.

    Raises
    ------
    ValueError
        If the value is not a valid boolean.
    """
    ...

def validate_axis_style_args(data, args, kwargs, arg_name, method_name) -> dict[str, Any]:
    """
    Argument handler for mixed index, columns / axis functions

    In an attempt to handle both `.method(index, columns)`, and
    `.method(arg, axis=.)`, we have to do some bad things to argument
    parsing. This translates all arguments to `{index=., columns=.}` style.

    Parameters
    ----------
    data : DataFrame
    args : tuple
        All positional arguments from the user
    kwargs : dict
        All keyword arguments from the user
    arg_name, method_name : str
        Used for better error messages

    Returns
    -------
    kwargs : dict
        A dictionary of keyword arguments. Doesn't modify ``kwargs``
        inplace, so update them with the return value here.

    Examples
    --------
    >>> df = pd.DataFrame(range(2))
    >>> validate_axis_style_args(df, (str.upper,), {'columns': id},
    ...                          'mapper', 'rename')
    {'columns': <built-in function id>, 'index': <method 'upper' of 'str' objects>}

    This emits a warning
    >>> validate_axis_style_args(df, (str.upper, id), {},
    ...                          'mapper', 'rename')
    {'index': <method 'upper' of 'str' objects>, 'columns': <built-in function id>}
    """
    ...

def validate_fillna_kwargs(value, method, validate_scalar_dict_value: bool = ...): # -> tuple[Unknown, str | Unknown | None]:
    """
    Validate the keyword arguments to 'fillna'.

    This checks that exactly one of 'value' and 'method' is specified.
    If 'method' is specified, this validates that it's a valid method.

    Parameters
    ----------
    value, method : object
        The 'value' and 'method' keyword arguments for 'fillna'.
    validate_scalar_dict_value : bool, default True
        Whether to validate that 'value' is a scalar or dict. Specifically,
        validate that it is not a list or tuple.

    Returns
    -------
    value, method : object
    """
    ...

def validate_percentile(q: float | Iterable[float]) -> np.ndarray:
    """
    Validate percentiles (used by describe and quantile).

    This function checks if the given float or iterable of floats is a valid percentile
    otherwise raises a ValueError.

    Parameters
    ----------
    q: float or iterable of floats
        A single percentile or an iterable of percentiles.

    Returns
    -------
    ndarray
        An ndarray of the percentiles if valid.

    Raises
    ------
    ValueError if percentiles are not in given interval([0, 1]).
    """
    ...

@overload
def validate_ascending(ascending: BoolishT) -> BoolishT:
    ...

@overload
def validate_ascending(ascending: Sequence[BoolishT]) -> list[BoolishT]:
    ...

def validate_ascending(ascending: bool | int | Sequence[BoolishT]) -> bool | int | list[BoolishT]:
    """Validate ``ascending`` kwargs for ``sort_index`` method."""
    ...

def validate_endpoints(closed: str | None) -> tuple[bool, bool]:
    """
    Check that the `closed` argument is among [None, "left", "right"]

    Parameters
    ----------
    closed : {None, "left", "right"}

    Returns
    -------
    left_closed : bool
    right_closed : bool

    Raises
    ------
    ValueError : if argument is not among valid values
    """
    ...

def validate_inclusive(inclusive: str | None) -> tuple[bool, bool]:
    """
    Check that the `inclusive` argument is among {"both", "neither", "left", "right"}.

    Parameters
    ----------
    inclusive : {"both", "neither", "left", "right"}

    Returns
    -------
    left_right_inclusive : tuple[bool, bool]

    Raises
    ------
    ValueError : if argument is not among valid values
    """
    ...

def validate_insert_loc(loc: int, length: int) -> int:
    """
    Check that we have an integer between -length and length, inclusive.

    Standardize negative loc to within [0, length].

    The exceptions we raise on failure match np.insert.
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas._typing import Scalar, StorageOptions
from pandas.util._decorators import doc
from pandas.core.shared_docs import _shared_docs
from pandas.io.excel._base import BaseExcelReader

class XlrdReader(BaseExcelReader):
    @doc(storage_options=_shared_docs["storage_options"])
    def __init__(self, filepath_or_buffer, storage_options: StorageOptions = ...) -> None:
        """
        Reader using xlrd engine.

        Parameters
        ----------
        filepath_or_buffer : str, path object or Workbook
            Object to be parsed.
        {storage_options}
        """
        ...
    
    def load_workbook(self, filepath_or_buffer):
        ...
    
    @property
    def sheet_names(self):
        ...
    
    def get_sheet_by_name(self, name):
        ...
    
    def get_sheet_by_index(self, index):
        ...
    
    def get_sheet_data(self, sheet, convert_float: bool, file_rows_needed: int | None = ...) -> list[list[Scalar]]:
        ...
    





"""
This type stub file was generated by pyright.
"""

from pandas.util._decorators import Appender, Substitution, cache_readonly
from pandas.core.util.hashing import hash_array, hash_pandas_object

def __getattr__(name): # -> Module("pandas.util.testing.util.testing"):
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from datetime import datetime, tzinfo

DT64NS_DTYPE: np.dtype
TD64NS_DTYPE: np.dtype
def precision_from_unit(unit: str) -> tuple[int, int]:
    ...

def localize_pydatetime(dt: datetime, tz: tzinfo | None) -> datetime:
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.indexes.base import Index
from pandas.core.indexes.range import RangeIndex

_sort_msg = ...
__all__ = ["Index", "MultiIndex", "NumericIndex", "Float64Index", "Int64Index", "CategoricalIndex", "IntervalIndex", "RangeIndex", "UInt64Index", "InvalidIndexError", "TimedeltaIndex", "PeriodIndex", "DatetimeIndex", "_new_Index", "NaT", "ensure_index", "ensure_index_from_sequences", "get_objs_combined_axis", "union_indexes", "get_unanimous_names", "all_indexes_same", "default_index", "safe_sort_index"]
def get_objs_combined_axis(objs, intersect: bool = ..., axis=..., sort: bool = ..., copy: bool = ...) -> Index:
    """
    Extract combined index: return intersection or union (depending on the
    value of "intersect") of indexes on given axis, or None if all objects
    lack indexes (e.g. they are numpy arrays).

    Parameters
    ----------
    objs : list
        Series or DataFrame objects, may be mix of the two.
    intersect : bool, default False
        If True, calculate the intersection between indexes. Otherwise,
        calculate the union.
    axis : {0 or 'index', 1 or 'outer'}, default 0
        The axis to extract indexes from.
    sort : bool, default True
        Whether the result index should come out sorted or not.
    copy : bool, default False
        If True, return a copy of the combined index.

    Returns
    -------
    Index
    """
    ...

def safe_sort_index(index: Index) -> Index:
    """
    Returns the sorted index

    We keep the dtypes and the name attributes.

    Parameters
    ----------
    index : an Index

    Returns
    -------
    Index
    """
    ...

def union_indexes(indexes, sort: bool | None = ...) -> Index:
    """
    Return the union of indexes.

    The behavior of sort and names is not consistent.

    Parameters
    ----------
    indexes : list of Index or list objects
    sort : bool, default True
        Whether the result index should come out sorted or not.

    Returns
    -------
    Index
    """
    ...

def all_indexes_same(indexes) -> bool:
    """
    Determine if all indexes contain the same elements.

    Parameters
    ----------
    indexes : iterable of Index objects

    Returns
    -------
    bool
        True if all indexes contain the same elements, False otherwise.
    """
    ...

def default_index(n: int) -> RangeIndex:
    ...




"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas.util._decorators import Appender, deprecate_kwarg
from pandas.core.shared_docs import _shared_docs
from pandas import DataFrame

if TYPE_CHECKING:
    ...
@Appender(_shared_docs["melt"] % { "caller": "pd.melt(df, ","other": "DataFrame.melt" })
def melt(frame: DataFrame, id_vars=..., value_vars=..., var_name=..., value_name=..., col_level=..., ignore_index: bool = ...) -> DataFrame:
    ...

@deprecate_kwarg(old_arg_name="label", new_arg_name=None)
def lreshape(data: DataFrame, groups, dropna: bool = ..., label=...) -> DataFrame:
    """
    Reshape wide-format data to long. Generalized inverse of DataFrame.pivot.

    Accepts a dictionary, ``groups``, in which each key is a new column name
    and each value is a list of old column names that will be "melted" under
    the new column name as part of the reshape.

    Parameters
    ----------
    data : DataFrame
        The wide-format DataFrame.
    groups : dict
        {new_name : list_of_columns}.
    dropna : bool, default True
        Do not include columns whose entries are all NaN.
    label : None
        Not used.

        .. deprecated:: 1.0.0

    Returns
    -------
    DataFrame
        Reshaped DataFrame.

    See Also
    --------
    melt : Unpivot a DataFrame from wide to long format, optionally leaving
        identifiers set.
    pivot : Create a spreadsheet-style pivot table as a DataFrame.
    DataFrame.pivot : Pivot without aggregation that can handle
        non-numeric data.
    DataFrame.pivot_table : Generalization of pivot that can handle
        duplicate values for one index/column pair.
    DataFrame.unstack : Pivot based on the index values instead of a
        column.
    wide_to_long : Wide panel to long format. Less flexible but more
        user-friendly than melt.

    Examples
    --------
    >>> data = pd.DataFrame({'hr1': [514, 573], 'hr2': [545, 526],
    ...                      'team': ['Red Sox', 'Yankees'],
    ...                      'year1': [2007, 2007], 'year2': [2008, 2008]})
    >>> data
       hr1  hr2     team  year1  year2
    0  514  545  Red Sox   2007   2008
    1  573  526  Yankees   2007   2008

    >>> pd.lreshape(data, {'year': ['year1', 'year2'], 'hr': ['hr1', 'hr2']})
          team  year   hr
    0  Red Sox  2007  514
    1  Yankees  2007  573
    2  Red Sox  2008  545
    3  Yankees  2008  526
    """
    ...

def wide_to_long(df: DataFrame, stubnames, i, j, sep: str = ..., suffix: str = ...) -> DataFrame:
    r"""
    Unpivot a DataFrame from wide to long format.

    Less flexible but more user-friendly than melt.

    With stubnames ['A', 'B'], this function expects to find one or more
    group of columns with format
    A-suffix1, A-suffix2,..., B-suffix1, B-suffix2,...
    You specify what you want to call this suffix in the resulting long format
    with `j` (for example `j='year'`)

    Each row of these wide variables are assumed to be uniquely identified by
    `i` (can be a single column name or a list of column names)

    All remaining variables in the data frame are left intact.

    Parameters
    ----------
    df : DataFrame
        The wide-format DataFrame.
    stubnames : str or list-like
        The stub name(s). The wide format variables are assumed to
        start with the stub names.
    i : str or list-like
        Column(s) to use as id variable(s).
    j : str
        The name of the sub-observation variable. What you wish to name your
        suffix in the long format.
    sep : str, default ""
        A character indicating the separation of the variable names
        in the wide format, to be stripped from the names in the long format.
        For example, if your column names are A-suffix1, A-suffix2, you
        can strip the hyphen by specifying `sep='-'`.
    suffix : str, default '\\d+'
        A regular expression capturing the wanted suffixes. '\\d+' captures
        numeric suffixes. Suffixes with no numbers could be specified with the
        negated character class '\\D+'. You can also further disambiguate
        suffixes, for example, if your wide variables are of the form A-one,
        B-two,.., and you have an unrelated column A-rating, you can ignore the
        last one by specifying `suffix='(!?one|two)'`. When all suffixes are
        numeric, they are cast to int64/float64.

    Returns
    -------
    DataFrame
        A DataFrame that contains each stub name as a variable, with new index
        (i, j).

    See Also
    --------
    melt : Unpivot a DataFrame from wide to long format, optionally leaving
        identifiers set.
    pivot : Create a spreadsheet-style pivot table as a DataFrame.
    DataFrame.pivot : Pivot without aggregation that can handle
        non-numeric data.
    DataFrame.pivot_table : Generalization of pivot that can handle
        duplicate values for one index/column pair.
    DataFrame.unstack : Pivot based on the index values instead of a
        column.

    Notes
    -----
    All extra variables are left untouched. This simply uses
    `pandas.melt` under the hood, but is hard-coded to "do the right thing"
    in a typical case.

    Examples
    --------
    >>> np.random.seed(123)
    >>> df = pd.DataFrame({"A1970" : {0 : "a", 1 : "b", 2 : "c"},
    ...                    "A1980" : {0 : "d", 1 : "e", 2 : "f"},
    ...                    "B1970" : {0 : 2.5, 1 : 1.2, 2 : .7},
    ...                    "B1980" : {0 : 3.2, 1 : 1.3, 2 : .1},
    ...                    "X"     : dict(zip(range(3), np.random.randn(3)))
    ...                   })
    >>> df["id"] = df.index
    >>> df
      A1970 A1980  B1970  B1980         X  id
    0     a     d    2.5    3.2 -1.085631   0
    1     b     e    1.2    1.3  0.997345   1
    2     c     f    0.7    0.1  0.282978   2
    >>> pd.wide_to_long(df, ["A", "B"], i="id", j="year")
    ... # doctest: +NORMALIZE_WHITESPACE
                    X  A    B
    id year
    0  1970 -1.085631  a  2.5
    1  1970  0.997345  b  1.2
    2  1970  0.282978  c  0.7
    0  1980 -1.085631  d  3.2
    1  1980  0.997345  e  1.3
    2  1980  0.282978  f  0.1

    With multiple id columns

    >>> df = pd.DataFrame({
    ...     'famid': [1, 1, 1, 2, 2, 2, 3, 3, 3],
    ...     'birth': [1, 2, 3, 1, 2, 3, 1, 2, 3],
    ...     'ht1': [2.8, 2.9, 2.2, 2, 1.8, 1.9, 2.2, 2.3, 2.1],
    ...     'ht2': [3.4, 3.8, 2.9, 3.2, 2.8, 2.4, 3.3, 3.4, 2.9]
    ... })
    >>> df
       famid  birth  ht1  ht2
    0      1      1  2.8  3.4
    1      1      2  2.9  3.8
    2      1      3  2.2  2.9
    3      2      1  2.0  3.2
    4      2      2  1.8  2.8
    5      2      3  1.9  2.4
    6      3      1  2.2  3.3
    7      3      2  2.3  3.4
    8      3      3  2.1  2.9
    >>> l = pd.wide_to_long(df, stubnames='ht', i=['famid', 'birth'], j='age')
    >>> l
    ... # doctest: +NORMALIZE_WHITESPACE
                      ht
    famid birth age
    1     1     1    2.8
                2    3.4
          2     1    2.9
                2    3.8
          3     1    2.2
                2    2.9
    2     1     1    2.0
                2    3.2
          2     1    1.8
                2    2.8
          3     1    1.9
                2    2.4
    3     1     1    2.2
                2    3.3
          2     1    2.3
                2    3.4
          3     1    2.1
                2    2.9

    Going from long back to wide just takes some creative use of `unstack`

    >>> w = l.unstack()
    >>> w.columns = w.columns.map('{0[0]}{0[1]}'.format)
    >>> w.reset_index()
       famid  birth  ht1  ht2
    0      1      1  2.8  3.4
    1      1      2  2.9  3.8
    2      1      3  2.2  2.9
    3      2      1  2.0  3.2
    4      2      2  1.8  2.8
    5      2      3  1.9  2.4
    6      3      1  2.2  3.3
    7      3      2  2.3  3.4
    8      3      3  2.1  2.9

    Less wieldy column names are also handled

    >>> np.random.seed(0)
    >>> df = pd.DataFrame({'A(weekly)-2010': np.random.rand(3),
    ...                    'A(weekly)-2011': np.random.rand(3),
    ...                    'B(weekly)-2010': np.random.rand(3),
    ...                    'B(weekly)-2011': np.random.rand(3),
    ...                    'X' : np.random.randint(3, size=3)})
    >>> df['id'] = df.index
    >>> df # doctest: +NORMALIZE_WHITESPACE, +ELLIPSIS
       A(weekly)-2010  A(weekly)-2011  B(weekly)-2010  B(weekly)-2011  X  id
    0        0.548814        0.544883        0.437587        0.383442  0   0
    1        0.715189        0.423655        0.891773        0.791725  1   1
    2        0.602763        0.645894        0.963663        0.528895  1   2

    >>> pd.wide_to_long(df, ['A(weekly)', 'B(weekly)'], i='id',
    ...                 j='year', sep='-')
    ... # doctest: +NORMALIZE_WHITESPACE
             X  A(weekly)  B(weekly)
    id year
    0  2010  0   0.548814   0.437587
    1  2010  1   0.715189   0.891773
    2  2010  1   0.602763   0.963663
    0  2011  0   0.544883   0.383442
    1  2011  1   0.423655   0.791725
    2  2011  1   0.645894   0.528895

    If we have many columns, we could also use a regex to find our
    stubnames and pass that list on to wide_to_long

    >>> stubnames = sorted(
    ...     set([match[0] for match in df.columns.str.findall(
    ...         r'[A-B]\(.*\)').values if match != []])
    ... )
    >>> list(stubnames)
    ['A(weekly)', 'B(weekly)']

    All of the above examples have integers as suffixes. It is possible to
    have non-integers as suffixes.

    >>> df = pd.DataFrame({
    ...     'famid': [1, 1, 1, 2, 2, 2, 3, 3, 3],
    ...     'birth': [1, 2, 3, 1, 2, 3, 1, 2, 3],
    ...     'ht_one': [2.8, 2.9, 2.2, 2, 1.8, 1.9, 2.2, 2.3, 2.1],
    ...     'ht_two': [3.4, 3.8, 2.9, 3.2, 2.8, 2.4, 3.3, 3.4, 2.9]
    ... })
    >>> df
       famid  birth  ht_one  ht_two
    0      1      1     2.8     3.4
    1      1      2     2.9     3.8
    2      1      3     2.2     2.9
    3      2      1     2.0     3.2
    4      2      2     1.8     2.8
    5      2      3     1.9     2.4
    6      3      1     2.2     3.3
    7      3      2     2.3     3.4
    8      3      3     2.1     2.9

    >>> l = pd.wide_to_long(df, stubnames='ht', i=['famid', 'birth'], j='age',
    ...                     sep='_', suffix=r'\w+')
    >>> l
    ... # doctest: +NORMALIZE_WHITESPACE
                      ht
    famid birth age
    1     1     one  2.8
                two  3.4
          2     one  2.9
                two  3.8
          3     one  2.2
                two  2.9
    2     1     one  2.0
                two  3.2
          2     one  1.8
                two  2.8
          3     one  1.9
                two  2.4
    3     1     one  2.2
                two  3.3
          2     one  2.3
                two  3.4
          3     one  2.1
                two  2.9
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.window.ewm import ExponentialMovingWindow, ExponentialMovingWindowGroupby
from pandas.core.window.expanding import Expanding, ExpandingGroupby
from pandas.core.window.rolling import Rolling, RollingGroupby, Window

__all__ = ["Expanding", "ExpandingGroupby", "ExponentialMovingWindow", "ExponentialMovingWindowGroupby", "Rolling", "RollingGroupby", "Window"]
